"""Implementations for common converter types"""
import ctypes,logging
from OpenGL._bytes import bytes, unicode, as_8_bit
from OpenGL._null import NULL
_log = logging.getLogger( 'OpenGL.converters' )

class Converter( object ):
    """Base class for Converter types

    Converter objects are callable objects used with the
    OpenGL.wrapper.Wrapper class to simplify the wrapping
    of functions by collecting commonly used functionality
    into a reusable function.

    Each Converter has two (class) attributes:

        argNames -- list of argument names for initialisation
        indexLookups -- set of (indexname, argName,methodName) values
            to lookup on wrapper.  These allow us to use argument-name
            references to refer to which arguments to use when
            processing (prevents the need to revise pointers when
            we change the API for a function).

    Converters can be any of the Wrapper API helper functions,
    so the callable interface can vary among Converter classes.
    """
    argNames = ( )
    indexLookups = ( )
    def __init__( self, *args, **named ):
        """Store arguments in attributes

        *args -- mapped to self.argNames in order to set attributes
        **named -- mapped to self.argNames by name to set attributes
        """
        argNames = list(self.argNames)
        for a in self.argNames:
            if a in named:
                setattr( self, a, named[a] )
                argNames.remove( a )
        for a,value in zip( argNames, args ):
            setattr( self, a, value )
    def finalise( self, wrapper ):
        """Look up our indices (where appropriate)"""
        for indexname,argName,methodName in self.indexLookups:
            setattr(
                self, indexname,
                getattr(wrapper,methodName)(getattr( self, argName ))
            )

# Definitions of the abstract interfaces...
class PyConverter( Converter ):
    """Converter sub-class for use in Wrapper.pyConverters

    This class just defines the interface for a pyConverter-style
    Converter object
    """
    def __call__( self, incoming, function, arguments ):
        """Convert incoming argument into compatable data-types

        incoming -- the Python argument for this parameter
        function -- the wrapper.Wrapper class we are supporting
        arguments -- the complete set of arguments passed to the
            function


        """
        raise NotImplemented( """%s class doesn't implement __call__"""%(
            self.__class__.__name__,
        ))

class CConverter( Converter ):
    """Converter sub-class for use in Wrapper.cConverters

    This class just defines the interface for a cConverter-style
    Converter object
    """
    def __call__( self, pyArgs, index, baseOperation ):
        """Calculate C-compatible Python object from Python arguments

        pyArgs -- set of Python argument objects converted by
            pyConverters from the incoming arguments
        index -- our index in baseOperation.cConverters
        baseOperation -- the Wrapper object which we are supporting
        """
        raise NotImplemented( """%s class doesn't implement __call__"""%(
            self.__class__.__name__,
        ))
class ReturnValues( Converter ):
    """Converter sub-class for use as Wrapper.returnValues

    This class just defines the interface for a returnValues-style
    Converter object
    """
    def __call__( self, result, baseOperation, pyArgs, cArgs ):
        """Return a final value to the caller

        result -- the raw ctypes result value
        baseOperation -- the Wrapper object which we are supporting
        pyArgs -- the set of Python arguments produced by pyConverters
        cArgs -- the set of C-compatible arguments produced by CConverter

        return the Python object for the final result
        """
        raise NotImplemented( """%s class doesn't implement __call__"""%(
            self.__class__.__name__,
        ))

# Now the concrete classes...
from OpenGL import acceleratesupport
CallFuncPyConverter = None
if acceleratesupport.ACCELERATE_AVAILABLE:
    try:
        from OpenGL_accelerate.wrapper import (
            CallFuncPyConverter, DefaultCConverter, getPyArgsName,
        )
        from OpenGL_accelerate.arraydatatype import (
            Output,SizedOutput,OutputOrInput,SizedOutputOrInput
        )
        from OpenGL_accelerate.wrapper import (
            returnCArgument, returnPyArgument,
        )
    except ImportError as err:
        _log.warning(
            "Unable to load converters accelerators (wrapper, arraydatatype) from OpenGL_accelerate"
        )
        CallFuncPyConverter = None
if CallFuncPyConverter is None:
    class CallFuncPyConverter( PyConverter ):
        """PyConverter that takes a callable and calls it on incoming"""
        def __init__( self, function ):
            """Store the function"""
            self.function = function
        def __call__( self, incoming, function, argument ):
            """Call our function on incoming"""
            return self.function( incoming )
    class DefaultCConverter( CConverter ):
        """NULL or Default CConverter, returns same-named Python argument

        Used primarily to allow for specifying a converter that explicitly
        says "use the default behaviour".  This is *not* a finalise-ing
        converter, it is passed in the index explicitly and just retrieves
        that index from pyArgs when called.

        Raises informative errors if the index cannot be resolved in pyArgs
        """
        def __init__( self, index ):
            """Just store index for future access"""
            self.index = index
        def __call__( self, pyArgs, index, wrapper ):
            """Return pyArgs[self.index] or raise a ValueError"""
            try:
                return pyArgs[ self.index ]
            except IndexError:
                raise ValueError(
                    """Expected parameter index %r, but pyArgs only length %s"""%(
                    self.index,
                    len(pyArgs )
                ))
    class getPyArgsName( CConverter ):
        """CConverter returning named Python argument

        Intended for use in cConverters, the function returned
        retrieves the named pyArg and returns it when called.
        """
        argNames = ('name',)
        indexLookups = [ ('index','name', 'pyArgIndex' ), ]
        __slots__ = ( 'index', 'name')
        def __call__( self, pyArgs, index, baseOperation ):
            """Return pyArgs[ self.index ]"""
            try:
                return pyArgs[ self.index ]
            except AttributeError:
                raise RuntimeError( """"Did not resolve parameter index for %r"""%(self.name))

    class Output( CConverter ):
        """CConverter generating static-size typed output arrays

        Produces an output array of given type (arrayType) and
        size using self.lookup() to determine the size of the
        array to be produced, where the lookup function is passed
        as an initialisation argument.

        Provides also:

            oldStyleReturn( ... ) for use in the default case of
                PyOpenGL compatability mode, where result arrays of
                size (1,) are returned as scalar values.
        """
        argNames = ('name','size','arrayType' )
        indexLookups = [
            ('outIndex','name', 'cArgIndex' ),
        ]
        __slots__ = ('index','size','arrayType','outIndex','inIndex')
        def __call__( self, pyArgs, index, baseOperation ):
            """Return pyArgs[ self.index ]"""
            return self.arrayType.zeros( self.getSize(pyArgs) )
        def getSize( self, pyArgs ):
            """Retrieve the array size for this argument"""
            return self.size
        def oldStyleReturn( self, result, baseOperation, pyArgs, cArgs ):
            """Retrieve cArgs[ self.index ]"""
            result = cArgs[ self.outIndex ]
            try:
                thisSize = self.getSize(pyArgs)
            except KeyError:
                return result 
            if thisSize == (1,):
                try:
                    return result[0]
                except (IndexError,TypeError):
                    return result
            else:
                return result
    class OutputOrInput( Output ):
        DO_OUTPUT = (None,NULL)
        def __call__( self, pyArgs, index, baseOperation ):
            for do_output in self.DO_OUTPUT:
                if pyArgs[index] is do_output:
                    return super( OutputOrInput,self ).__call__( pyArgs, index, baseOperation )
            return self.arrayType.asArray( pyArgs[index] )

    class SizedOutput( Output ):
        """Output generating dynamically-sized typed output arrays

        Takes an extra parameter "specifier", which is the name of
        a Python argument to be passed to the lookup function in order
        to determine the appropriate size for the output array.
        """
        argNames = ('name','specifier','lookup','arrayType' )
        indexLookups = [
            ('outIndex','name', 'cArgIndex' ),
            ('index','specifier', 'pyArgIndex' ),
        ]
        __slots__ = ('index','outIndex','specifier','lookup','arrayType')
        def getSize( self, pyArgs ):
            """Retrieve the array size for this argument"""
            try:
                specifier = pyArgs[ self.index ]
            except AttributeError:
                raise RuntimeError( """"Did not resolve parameter index for %r"""%(self.name))
            else:
                try:
                    return self.lookup( specifier )
                except KeyError:
                    raise KeyError( """Unknown specifier %s"""%( specifier ))
    class SizedOutputOrInput( SizedOutput ):
        DO_OUTPUT = (None,NULL)
        def __call__( self, pyArgs, index, baseOperation ):
            for do_output in self.DO_OUTPUT:
                if pyArgs[index] is do_output:
                    return super( SizedOutputOrInput,self ).__call__( pyArgs, index, baseOperation )
            return self.arrayType.asArray( pyArgs[index] )
    class returnCArgument( ReturnValues ):
        """ReturnValues returning the named cArgs value"""
        argNames = ('name',)
        indexLookups = [ ('index','name', 'cArgIndex' ), ]
        __slots__ = ( 'index', 'name' )
        def __call__( self, result, baseOperation, pyArgs, cArgs ):
            """Retrieve cArgs[ self.index ]"""
            return cArgs[self.index]

    class returnPyArgument( ReturnValues ):
        """ReturnValues returning the named pyArgs value"""
        argNames = ('name',)
        indexLookups = [ ('index','name', 'pyArgIndex' ), ]
        __slots__ = ( 'index', 'name' )
        def __call__( self, result, baseOperation, pyArgs, cArgs ):
            """Retrieve pyArgs[ self.index ]"""
            return pyArgs[self.index]

class StringLengths( CConverter ):
    """CConverter for processing array-of-pointers-to-strings data-type

    Converter is a CConverter for the array-of-lengths for a
    array-of-pointers-to-strings data-type used to pass a set
    of code fragments to the GLSL compiler.

    Provides also:

        stringArray -- PyConverter callable ensuring list-of-strings
            format for the python argument

        stringArrayForC -- CResolver converting the array to
            POINTER(c_char_p) format for passing to C

        totalCount -- CConverter callable giving count of string
            pointers (that is, length of the pointer array)
    """
    argNames = ('name',)
    indexLookups = [ ('index','name', 'pyArgIndex' ), ]
    __slots__ = ()
    def __call__( self, pyArgs, index, baseOperation ):
        """Get array of length integers for string contents"""
        from OpenGL.raw.GL import _types
        tmp = [len(x) for x in pyArgs[self.index]]
        a_type = _types.GLint * len(tmp)
        return a_type( *tmp )
    def totalCount( self, pyArgs, index, baseOperation ):
        """Get array of length integers for string contents"""
        return len(pyArgs[self.index])
    def stringArray( self, arg, baseOperation, args ):
        """Create basic array-of-strings object from pyArg"""
        if isinstance( arg, (bytes,unicode) ):
            arg = [arg]
        value = [as_8_bit(x) for x in arg]
        return value
    def stringArrayForC( self, strings ):
        """Create a ctypes pointer to char-pointer set"""
        from OpenGL import arrays
        result = (ctypes.c_char_p * len(strings))()
        for i,s in enumerate(strings):
            result[i] = ctypes.cast(
                arrays.GLcharARBArray.dataPointer(s),
                ctypes.c_char_p,
            )
        return result



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_VERSION_GL_4_3'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_VERSION_GL_4_3',error_checker=_errors._error_checker)
GL_ACTIVE_RESOURCES=_C('GL_ACTIVE_RESOURCES',0x92F5)
GL_ACTIVE_VARIABLES=_C('GL_ACTIVE_VARIABLES',0x9305)
GL_ANY_SAMPLES_PASSED_CONSERVATIVE=_C('GL_ANY_SAMPLES_PASSED_CONSERVATIVE',0x8D6A)
GL_ARRAY_SIZE=_C('GL_ARRAY_SIZE',0x92FB)
GL_ARRAY_STRIDE=_C('GL_ARRAY_STRIDE',0x92FE)
GL_ATOMIC_COUNTER_BUFFER_INDEX=_C('GL_ATOMIC_COUNTER_BUFFER_INDEX',0x9301)
GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_COMPUTE_SHADER=_C('GL_ATOMIC_COUNTER_BUFFER_REFERENCED_BY_COMPUTE_SHADER',0x90ED)
GL_AUTO_GENERATE_MIPMAP=_C('GL_AUTO_GENERATE_MIPMAP',0x8295)
GL_BLOCK_INDEX=_C('GL_BLOCK_INDEX',0x92FD)
GL_BUFFER=_C('GL_BUFFER',0x82E0)
GL_BUFFER_BINDING=_C('GL_BUFFER_BINDING',0x9302)
GL_BUFFER_DATA_SIZE=_C('GL_BUFFER_DATA_SIZE',0x9303)
GL_BUFFER_VARIABLE=_C('GL_BUFFER_VARIABLE',0x92E5)
GL_CAVEAT_SUPPORT=_C('GL_CAVEAT_SUPPORT',0x82B8)
GL_CLEAR_BUFFER=_C('GL_CLEAR_BUFFER',0x82B4)
GL_COLOR_COMPONENTS=_C('GL_COLOR_COMPONENTS',0x8283)
GL_COLOR_ENCODING=_C('GL_COLOR_ENCODING',0x8296)
GL_COLOR_RENDERABLE=_C('GL_COLOR_RENDERABLE',0x8286)
GL_COMPRESSED_R11_EAC=_C('GL_COMPRESSED_R11_EAC',0x9270)
GL_COMPRESSED_RG11_EAC=_C('GL_COMPRESSED_RG11_EAC',0x9272)
GL_COMPRESSED_RGB8_ETC2=_C('GL_COMPRESSED_RGB8_ETC2',0x9274)
GL_COMPRESSED_RGB8_PUNCHTHROUGH_ALPHA1_ETC2=_C('GL_COMPRESSED_RGB8_PUNCHTHROUGH_ALPHA1_ETC2',0x9276)
GL_COMPRESSED_RGBA8_ETC2_EAC=_C('GL_COMPRESSED_RGBA8_ETC2_EAC',0x9278)
GL_COMPRESSED_SIGNED_R11_EAC=_C('GL_COMPRESSED_SIGNED_R11_EAC',0x9271)
GL_COMPRESSED_SIGNED_RG11_EAC=_C('GL_COMPRESSED_SIGNED_RG11_EAC',0x9273)
GL_COMPRESSED_SRGB8_ALPHA8_ETC2_EAC=_C('GL_COMPRESSED_SRGB8_ALPHA8_ETC2_EAC',0x9279)
GL_COMPRESSED_SRGB8_ETC2=_C('GL_COMPRESSED_SRGB8_ETC2',0x9275)
GL_COMPRESSED_SRGB8_PUNCHTHROUGH_ALPHA1_ETC2=_C('GL_COMPRESSED_SRGB8_PUNCHTHROUGH_ALPHA1_ETC2',0x9277)
GL_COMPUTE_SHADER=_C('GL_COMPUTE_SHADER',0x91B9)
GL_COMPUTE_SHADER_BIT=_C('GL_COMPUTE_SHADER_BIT',0x00000020)
GL_COMPUTE_SUBROUTINE=_C('GL_COMPUTE_SUBROUTINE',0x92ED)
GL_COMPUTE_SUBROUTINE_UNIFORM=_C('GL_COMPUTE_SUBROUTINE_UNIFORM',0x92F3)
GL_COMPUTE_TEXTURE=_C('GL_COMPUTE_TEXTURE',0x82A0)
GL_COMPUTE_WORK_GROUP_SIZE=_C('GL_COMPUTE_WORK_GROUP_SIZE',0x8267)
GL_CONTEXT_FLAG_DEBUG_BIT=_C('GL_CONTEXT_FLAG_DEBUG_BIT',0x00000002)
GL_DEBUG_CALLBACK_FUNCTION=_C('GL_DEBUG_CALLBACK_FUNCTION',0x8244)
GL_DEBUG_CALLBACK_USER_PARAM=_C('GL_DEBUG_CALLBACK_USER_PARAM',0x8245)
GL_DEBUG_GROUP_STACK_DEPTH=_C('GL_DEBUG_GROUP_STACK_DEPTH',0x826D)
GL_DEBUG_LOGGED_MESSAGES=_C('GL_DEBUG_LOGGED_MESSAGES',0x9145)
GL_DEBUG_NEXT_LOGGED_MESSAGE_LENGTH=_C('GL_DEBUG_NEXT_LOGGED_MESSAGE_LENGTH',0x8243)
GL_DEBUG_OUTPUT=_C('GL_DEBUG_OUTPUT',0x92E0)
GL_DEBUG_OUTPUT_SYNCHRONOUS=_C('GL_DEBUG_OUTPUT_SYNCHRONOUS',0x8242)
GL_DEBUG_SEVERITY_HIGH=_C('GL_DEBUG_SEVERITY_HIGH',0x9146)
GL_DEBUG_SEVERITY_LOW=_C('GL_DEBUG_SEVERITY_LOW',0x9148)
GL_DEBUG_SEVERITY_MEDIUM=_C('GL_DEBUG_SEVERITY_MEDIUM',0x9147)
GL_DEBUG_SEVERITY_NOTIFICATION=_C('GL_DEBUG_SEVERITY_NOTIFICATION',0x826B)
GL_DEBUG_SOURCE_API=_C('GL_DEBUG_SOURCE_API',0x8246)
GL_DEBUG_SOURCE_APPLICATION=_C('GL_DEBUG_SOURCE_APPLICATION',0x824A)
GL_DEBUG_SOURCE_OTHER=_C('GL_DEBUG_SOURCE_OTHER',0x824B)
GL_DEBUG_SOURCE_SHADER_COMPILER=_C('GL_DEBUG_SOURCE_SHADER_COMPILER',0x8248)
GL_DEBUG_SOURCE_THIRD_PARTY=_C('GL_DEBUG_SOURCE_THIRD_PARTY',0x8249)
GL_DEBUG_SOURCE_WINDOW_SYSTEM=_C('GL_DEBUG_SOURCE_WINDOW_SYSTEM',0x8247)
GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR=_C('GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR',0x824D)
GL_DEBUG_TYPE_ERROR=_C('GL_DEBUG_TYPE_ERROR',0x824C)
GL_DEBUG_TYPE_MARKER=_C('GL_DEBUG_TYPE_MARKER',0x8268)
GL_DEBUG_TYPE_OTHER=_C('GL_DEBUG_TYPE_OTHER',0x8251)
GL_DEBUG_TYPE_PERFORMANCE=_C('GL_DEBUG_TYPE_PERFORMANCE',0x8250)
GL_DEBUG_TYPE_POP_GROUP=_C('GL_DEBUG_TYPE_POP_GROUP',0x826A)
GL_DEBUG_TYPE_PORTABILITY=_C('GL_DEBUG_TYPE_PORTABILITY',0x824F)
GL_DEBUG_TYPE_PUSH_GROUP=_C('GL_DEBUG_TYPE_PUSH_GROUP',0x8269)
GL_DEBUG_TYPE_UNDEFINED_BEHAVIOR=_C('GL_DEBUG_TYPE_UNDEFINED_BEHAVIOR',0x824E)
GL_DEPTH_COMPONENTS=_C('GL_DEPTH_COMPONENTS',0x8284)
GL_DEPTH_RENDERABLE=_C('GL_DEPTH_RENDERABLE',0x8287)
GL_DEPTH_STENCIL_TEXTURE_MODE=_C('GL_DEPTH_STENCIL_TEXTURE_MODE',0x90EA)
GL_DISPATCH_INDIRECT_BUFFER=_C('GL_DISPATCH_INDIRECT_BUFFER',0x90EE)
GL_DISPATCH_INDIRECT_BUFFER_BINDING=_C('GL_DISPATCH_INDIRECT_BUFFER_BINDING',0x90EF)
GL_DISPLAY_LIST=_C('GL_DISPLAY_LIST',0x82E7)
GL_FILTER=_C('GL_FILTER',0x829A)
GL_FRAGMENT_SUBROUTINE=_C('GL_FRAGMENT_SUBROUTINE',0x92EC)
GL_FRAGMENT_SUBROUTINE_UNIFORM=_C('GL_FRAGMENT_SUBROUTINE_UNIFORM',0x92F2)
GL_FRAGMENT_TEXTURE=_C('GL_FRAGMENT_TEXTURE',0x829F)
GL_FRAMEBUFFER_BLEND=_C('GL_FRAMEBUFFER_BLEND',0x828B)
GL_FRAMEBUFFER_DEFAULT_FIXED_SAMPLE_LOCATIONS=_C('GL_FRAMEBUFFER_DEFAULT_FIXED_SAMPLE_LOCATIONS',0x9314)
GL_FRAMEBUFFER_DEFAULT_HEIGHT=_C('GL_FRAMEBUFFER_DEFAULT_HEIGHT',0x9311)
GL_FRAMEBUFFER_DEFAULT_LAYERS=_C('GL_FRAMEBUFFER_DEFAULT_LAYERS',0x9312)
GL_FRAMEBUFFER_DEFAULT_SAMPLES=_C('GL_FRAMEBUFFER_DEFAULT_SAMPLES',0x9313)
GL_FRAMEBUFFER_DEFAULT_WIDTH=_C('GL_FRAMEBUFFER_DEFAULT_WIDTH',0x9310)
GL_FRAMEBUFFER_RENDERABLE=_C('GL_FRAMEBUFFER_RENDERABLE',0x8289)
GL_FRAMEBUFFER_RENDERABLE_LAYERED=_C('GL_FRAMEBUFFER_RENDERABLE_LAYERED',0x828A)
GL_FULL_SUPPORT=_C('GL_FULL_SUPPORT',0x82B7)
GL_GEOMETRY_SUBROUTINE=_C('GL_GEOMETRY_SUBROUTINE',0x92EB)
GL_GEOMETRY_SUBROUTINE_UNIFORM=_C('GL_GEOMETRY_SUBROUTINE_UNIFORM',0x92F1)
GL_GEOMETRY_TEXTURE=_C('GL_GEOMETRY_TEXTURE',0x829E)
GL_GET_TEXTURE_IMAGE_FORMAT=_C('GL_GET_TEXTURE_IMAGE_FORMAT',0x8291)
GL_GET_TEXTURE_IMAGE_TYPE=_C('GL_GET_TEXTURE_IMAGE_TYPE',0x8292)
GL_IMAGE_CLASS_10_10_10_2=_C('GL_IMAGE_CLASS_10_10_10_2',0x82C3)
GL_IMAGE_CLASS_11_11_10=_C('GL_IMAGE_CLASS_11_11_10',0x82C2)
GL_IMAGE_CLASS_1_X_16=_C('GL_IMAGE_CLASS_1_X_16',0x82BE)
GL_IMAGE_CLASS_1_X_32=_C('GL_IMAGE_CLASS_1_X_32',0x82BB)
GL_IMAGE_CLASS_1_X_8=_C('GL_IMAGE_CLASS_1_X_8',0x82C1)
GL_IMAGE_CLASS_2_X_16=_C('GL_IMAGE_CLASS_2_X_16',0x82BD)
GL_IMAGE_CLASS_2_X_32=_C('GL_IMAGE_CLASS_2_X_32',0x82BA)
GL_IMAGE_CLASS_2_X_8=_C('GL_IMAGE_CLASS_2_X_8',0x82C0)
GL_IMAGE_CLASS_4_X_16=_C('GL_IMAGE_CLASS_4_X_16',0x82BC)
GL_IMAGE_CLASS_4_X_32=_C('GL_IMAGE_CLASS_4_X_32',0x82B9)
GL_IMAGE_CLASS_4_X_8=_C('GL_IMAGE_CLASS_4_X_8',0x82BF)
GL_IMAGE_COMPATIBILITY_CLASS=_C('GL_IMAGE_COMPATIBILITY_CLASS',0x82A8)
GL_IMAGE_PIXEL_FORMAT=_C('GL_IMAGE_PIXEL_FORMAT',0x82A9)
GL_IMAGE_PIXEL_TYPE=_C('GL_IMAGE_PIXEL_TYPE',0x82AA)
GL_IMAGE_TEXEL_SIZE=_C('GL_IMAGE_TEXEL_SIZE',0x82A7)
GL_INTERNALFORMAT_ALPHA_SIZE=_C('GL_INTERNALFORMAT_ALPHA_SIZE',0x8274)
GL_INTERNALFORMAT_ALPHA_TYPE=_C('GL_INTERNALFORMAT_ALPHA_TYPE',0x827B)
GL_INTERNALFORMAT_BLUE_SIZE=_C('GL_INTERNALFORMAT_BLUE_SIZE',0x8273)
GL_INTERNALFORMAT_BLUE_TYPE=_C('GL_INTERNALFORMAT_BLUE_TYPE',0x827A)
GL_INTERNALFORMAT_DEPTH_SIZE=_C('GL_INTERNALFORMAT_DEPTH_SIZE',0x8275)
GL_INTERNALFORMAT_DEPTH_TYPE=_C('GL_INTERNALFORMAT_DEPTH_TYPE',0x827C)
GL_INTERNALFORMAT_GREEN_SIZE=_C('GL_INTERNALFORMAT_GREEN_SIZE',0x8272)
GL_INTERNALFORMAT_GREEN_TYPE=_C('GL_INTERNALFORMAT_GREEN_TYPE',0x8279)
GL_INTERNALFORMAT_PREFERRED=_C('GL_INTERNALFORMAT_PREFERRED',0x8270)
GL_INTERNALFORMAT_RED_SIZE=_C('GL_INTERNALFORMAT_RED_SIZE',0x8271)
GL_INTERNALFORMAT_RED_TYPE=_C('GL_INTERNALFORMAT_RED_TYPE',0x8278)
GL_INTERNALFORMAT_SHARED_SIZE=_C('GL_INTERNALFORMAT_SHARED_SIZE',0x8277)
GL_INTERNALFORMAT_STENCIL_SIZE=_C('GL_INTERNALFORMAT_STENCIL_SIZE',0x8276)
GL_INTERNALFORMAT_STENCIL_TYPE=_C('GL_INTERNALFORMAT_STENCIL_TYPE',0x827D)
GL_INTERNALFORMAT_SUPPORTED=_C('GL_INTERNALFORMAT_SUPPORTED',0x826F)
GL_IS_PER_PATCH=_C('GL_IS_PER_PATCH',0x92E7)
GL_IS_ROW_MAJOR=_C('GL_IS_ROW_MAJOR',0x9300)
GL_LOCATION=_C('GL_LOCATION',0x930E)
GL_LOCATION_INDEX=_C('GL_LOCATION_INDEX',0x930F)
GL_MANUAL_GENERATE_MIPMAP=_C('GL_MANUAL_GENERATE_MIPMAP',0x8294)
GL_MATRIX_STRIDE=_C('GL_MATRIX_STRIDE',0x92FF)
GL_MAX_COMBINED_COMPUTE_UNIFORM_COMPONENTS=_C('GL_MAX_COMBINED_COMPUTE_UNIFORM_COMPONENTS',0x8266)
GL_MAX_COMBINED_DIMENSIONS=_C('GL_MAX_COMBINED_DIMENSIONS',0x8282)
GL_MAX_COMBINED_SHADER_OUTPUT_RESOURCES=_C('GL_MAX_COMBINED_SHADER_OUTPUT_RESOURCES',0x8F39)
GL_MAX_COMBINED_SHADER_STORAGE_BLOCKS=_C('GL_MAX_COMBINED_SHADER_STORAGE_BLOCKS',0x90DC)
GL_MAX_COMPUTE_ATOMIC_COUNTERS=_C('GL_MAX_COMPUTE_ATOMIC_COUNTERS',0x8265)
GL_MAX_COMPUTE_ATOMIC_COUNTER_BUFFERS=_C('GL_MAX_COMPUTE_ATOMIC_COUNTER_BUFFERS',0x8264)
GL_MAX_COMPUTE_IMAGE_UNIFORMS=_C('GL_MAX_COMPUTE_IMAGE_UNIFORMS',0x91BD)
GL_MAX_COMPUTE_SHADER_STORAGE_BLOCKS=_C('GL_MAX_COMPUTE_SHADER_STORAGE_BLOCKS',0x90DB)
GL_MAX_COMPUTE_SHARED_MEMORY_SIZE=_C('GL_MAX_COMPUTE_SHARED_MEMORY_SIZE',0x8262)
GL_MAX_COMPUTE_TEXTURE_IMAGE_UNITS=_C('GL_MAX_COMPUTE_TEXTURE_IMAGE_UNITS',0x91BC)
GL_MAX_COMPUTE_UNIFORM_BLOCKS=_C('GL_MAX_COMPUTE_UNIFORM_BLOCKS',0x91BB)
GL_MAX_COMPUTE_UNIFORM_COMPONENTS=_C('GL_MAX_COMPUTE_UNIFORM_COMPONENTS',0x8263)
GL_MAX_COMPUTE_WORK_GROUP_COUNT=_C('GL_MAX_COMPUTE_WORK_GROUP_COUNT',0x91BE)
GL_MAX_COMPUTE_WORK_GROUP_INVOCATIONS=_C('GL_MAX_COMPUTE_WORK_GROUP_INVOCATIONS',0x90EB)
GL_MAX_COMPUTE_WORK_GROUP_SIZE=_C('GL_MAX_COMPUTE_WORK_GROUP_SIZE',0x91BF)
GL_MAX_DEBUG_GROUP_STACK_DEPTH=_C('GL_MAX_DEBUG_GROUP_STACK_DEPTH',0x826C)
GL_MAX_DEBUG_LOGGED_MESSAGES=_C('GL_MAX_DEBUG_LOGGED_MESSAGES',0x9144)
GL_MAX_DEBUG_MESSAGE_LENGTH=_C('GL_MAX_DEBUG_MESSAGE_LENGTH',0x9143)
GL_MAX_DEPTH=_C('GL_MAX_DEPTH',0x8280)
GL_MAX_ELEMENT_INDEX=_C('GL_MAX_ELEMENT_INDEX',0x8D6B)
GL_MAX_FRAGMENT_SHADER_STORAGE_BLOCKS=_C('GL_MAX_FRAGMENT_SHADER_STORAGE_BLOCKS',0x90DA)
GL_MAX_FRAMEBUFFER_HEIGHT=_C('GL_MAX_FRAMEBUFFER_HEIGHT',0x9316)
GL_MAX_FRAMEBUFFER_LAYERS=_C('GL_MAX_FRAMEBUFFER_LAYERS',0x9317)
GL_MAX_FRAMEBUFFER_SAMPLES=_C('GL_MAX_FRAMEBUFFER_SAMPLES',0x9318)
GL_MAX_FRAMEBUFFER_WIDTH=_C('GL_MAX_FRAMEBUFFER_WIDTH',0x9315)
GL_MAX_GEOMETRY_SHADER_STORAGE_BLOCKS=_C('GL_MAX_GEOMETRY_SHADER_STORAGE_BLOCKS',0x90D7)
GL_MAX_HEIGHT=_C('GL_MAX_HEIGHT',0x827F)
GL_MAX_LABEL_LENGTH=_C('GL_MAX_LABEL_LENGTH',0x82E8)
GL_MAX_LAYERS=_C('GL_MAX_LAYERS',0x8281)
GL_MAX_NAME_LENGTH=_C('GL_MAX_NAME_LENGTH',0x92F6)
GL_MAX_NUM_ACTIVE_VARIABLES=_C('GL_MAX_NUM_ACTIVE_VARIABLES',0x92F7)
GL_MAX_NUM_COMPATIBLE_SUBROUTINES=_C('GL_MAX_NUM_COMPATIBLE_SUBROUTINES',0x92F8)
GL_MAX_SHADER_STORAGE_BLOCK_SIZE=_C('GL_MAX_SHADER_STORAGE_BLOCK_SIZE',0x90DE)
GL_MAX_SHADER_STORAGE_BUFFER_BINDINGS=_C('GL_MAX_SHADER_STORAGE_BUFFER_BINDINGS',0x90DD)
GL_MAX_TESS_CONTROL_SHADER_STORAGE_BLOCKS=_C('GL_MAX_TESS_CONTROL_SHADER_STORAGE_BLOCKS',0x90D8)
GL_MAX_TESS_EVALUATION_SHADER_STORAGE_BLOCKS=_C('GL_MAX_TESS_EVALUATION_SHADER_STORAGE_BLOCKS',0x90D9)
GL_MAX_UNIFORM_LOCATIONS=_C('GL_MAX_UNIFORM_LOCATIONS',0x826E)
GL_MAX_VERTEX_ATTRIB_BINDINGS=_C('GL_MAX_VERTEX_ATTRIB_BINDINGS',0x82DA)
GL_MAX_VERTEX_ATTRIB_RELATIVE_OFFSET=_C('GL_MAX_VERTEX_ATTRIB_RELATIVE_OFFSET',0x82D9)
GL_MAX_VERTEX_SHADER_STORAGE_BLOCKS=_C('GL_MAX_VERTEX_SHADER_STORAGE_BLOCKS',0x90D6)
GL_MAX_WIDTH=_C('GL_MAX_WIDTH',0x827E)
GL_MIPMAP=_C('GL_MIPMAP',0x8293)
GL_NAME_LENGTH=_C('GL_NAME_LENGTH',0x92F9)
GL_NUM_ACTIVE_VARIABLES=_C('GL_NUM_ACTIVE_VARIABLES',0x9304)
GL_NUM_SHADING_LANGUAGE_VERSIONS=_C('GL_NUM_SHADING_LANGUAGE_VERSIONS',0x82E9)
GL_OFFSET=_C('GL_OFFSET',0x92FC)
GL_PRIMITIVE_RESTART_FIXED_INDEX=_C('GL_PRIMITIVE_RESTART_FIXED_INDEX',0x8D69)
GL_PROGRAM=_C('GL_PROGRAM',0x82E2)
GL_PROGRAM_INPUT=_C('GL_PROGRAM_INPUT',0x92E3)
GL_PROGRAM_OUTPUT=_C('GL_PROGRAM_OUTPUT',0x92E4)
GL_PROGRAM_PIPELINE=_C('GL_PROGRAM_PIPELINE',0x82E4)
GL_QUERY=_C('GL_QUERY',0x82E3)
GL_READ_PIXELS=_C('GL_READ_PIXELS',0x828C)
GL_READ_PIXELS_FORMAT=_C('GL_READ_PIXELS_FORMAT',0x828D)
GL_READ_PIXELS_TYPE=_C('GL_READ_PIXELS_TYPE',0x828E)
GL_REFERENCED_BY_COMPUTE_SHADER=_C('GL_REFERENCED_BY_COMPUTE_SHADER',0x930B)
GL_REFERENCED_BY_FRAGMENT_SHADER=_C('GL_REFERENCED_BY_FRAGMENT_SHADER',0x930A)
GL_REFERENCED_BY_GEOMETRY_SHADER=_C('GL_REFERENCED_BY_GEOMETRY_SHADER',0x9309)
GL_REFERENCED_BY_TESS_CONTROL_SHADER=_C('GL_REFERENCED_BY_TESS_CONTROL_SHADER',0x9307)
GL_REFERENCED_BY_TESS_EVALUATION_SHADER=_C('GL_REFERENCED_BY_TESS_EVALUATION_SHADER',0x9308)
GL_REFERENCED_BY_VERTEX_SHADER=_C('GL_REFERENCED_BY_VERTEX_SHADER',0x9306)
GL_SAMPLER=_C('GL_SAMPLER',0x82E6)
GL_SHADER=_C('GL_SHADER',0x82E1)
GL_SHADER_IMAGE_ATOMIC=_C('GL_SHADER_IMAGE_ATOMIC',0x82A6)
GL_SHADER_IMAGE_LOAD=_C('GL_SHADER_IMAGE_LOAD',0x82A4)
GL_SHADER_IMAGE_STORE=_C('GL_SHADER_IMAGE_STORE',0x82A5)
GL_SHADER_STORAGE_BARRIER_BIT=_C('GL_SHADER_STORAGE_BARRIER_BIT',0x00002000)
GL_SHADER_STORAGE_BLOCK=_C('GL_SHADER_STORAGE_BLOCK',0x92E6)
GL_SHADER_STORAGE_BUFFER=_C('GL_SHADER_STORAGE_BUFFER',0x90D2)
GL_SHADER_STORAGE_BUFFER_BINDING=_C('GL_SHADER_STORAGE_BUFFER_BINDING',0x90D3)
GL_SHADER_STORAGE_BUFFER_OFFSET_ALIGNMENT=_C('GL_SHADER_STORAGE_BUFFER_OFFSET_ALIGNMENT',0x90DF)
GL_SHADER_STORAGE_BUFFER_SIZE=_C('GL_SHADER_STORAGE_BUFFER_SIZE',0x90D5)
GL_SHADER_STORAGE_BUFFER_START=_C('GL_SHADER_STORAGE_BUFFER_START',0x90D4)
GL_SIMULTANEOUS_TEXTURE_AND_DEPTH_TEST=_C('GL_SIMULTANEOUS_TEXTURE_AND_DEPTH_TEST',0x82AC)
GL_SIMULTANEOUS_TEXTURE_AND_DEPTH_WRITE=_C('GL_SIMULTANEOUS_TEXTURE_AND_DEPTH_WRITE',0x82AE)
GL_SIMULTANEOUS_TEXTURE_AND_STENCIL_TEST=_C('GL_SIMULTANEOUS_TEXTURE_AND_STENCIL_TEST',0x82AD)
GL_SIMULTANEOUS_TEXTURE_AND_STENCIL_WRITE=_C('GL_SIMULTANEOUS_TEXTURE_AND_STENCIL_WRITE',0x82AF)
GL_SRGB_READ=_C('GL_SRGB_READ',0x8297)
GL_SRGB_WRITE=_C('GL_SRGB_WRITE',0x8298)
GL_STACK_OVERFLOW=_C('GL_STACK_OVERFLOW',0x0503)
GL_STACK_UNDERFLOW=_C('GL_STACK_UNDERFLOW',0x0504)
GL_STENCIL_COMPONENTS=_C('GL_STENCIL_COMPONENTS',0x8285)
GL_STENCIL_RENDERABLE=_C('GL_STENCIL_RENDERABLE',0x8288)
GL_TESS_CONTROL_SUBROUTINE=_C('GL_TESS_CONTROL_SUBROUTINE',0x92E9)
GL_TESS_CONTROL_SUBROUTINE_UNIFORM=_C('GL_TESS_CONTROL_SUBROUTINE_UNIFORM',0x92EF)
GL_TESS_CONTROL_TEXTURE=_C('GL_TESS_CONTROL_TEXTURE',0x829C)
GL_TESS_EVALUATION_SUBROUTINE=_C('GL_TESS_EVALUATION_SUBROUTINE',0x92EA)
GL_TESS_EVALUATION_SUBROUTINE_UNIFORM=_C('GL_TESS_EVALUATION_SUBROUTINE_UNIFORM',0x92F0)
GL_TESS_EVALUATION_TEXTURE=_C('GL_TESS_EVALUATION_TEXTURE',0x829D)
GL_TEXTURE_BUFFER_OFFSET=_C('GL_TEXTURE_BUFFER_OFFSET',0x919D)
GL_TEXTURE_BUFFER_OFFSET_ALIGNMENT=_C('GL_TEXTURE_BUFFER_OFFSET_ALIGNMENT',0x919F)
GL_TEXTURE_BUFFER_SIZE=_C('GL_TEXTURE_BUFFER_SIZE',0x919E)
GL_TEXTURE_COMPRESSED_BLOCK_HEIGHT=_C('GL_TEXTURE_COMPRESSED_BLOCK_HEIGHT',0x82B2)
GL_TEXTURE_COMPRESSED_BLOCK_SIZE=_C('GL_TEXTURE_COMPRESSED_BLOCK_SIZE',0x82B3)
GL_TEXTURE_COMPRESSED_BLOCK_WIDTH=_C('GL_TEXTURE_COMPRESSED_BLOCK_WIDTH',0x82B1)
GL_TEXTURE_GATHER=_C('GL_TEXTURE_GATHER',0x82A2)
GL_TEXTURE_GATHER_SHADOW=_C('GL_TEXTURE_GATHER_SHADOW',0x82A3)
GL_TEXTURE_IMAGE_FORMAT=_C('GL_TEXTURE_IMAGE_FORMAT',0x828F)
GL_TEXTURE_IMAGE_TYPE=_C('GL_TEXTURE_IMAGE_TYPE',0x8290)
GL_TEXTURE_IMMUTABLE_LEVELS=_C('GL_TEXTURE_IMMUTABLE_LEVELS',0x82DF)
GL_TEXTURE_SHADOW=_C('GL_TEXTURE_SHADOW',0x82A1)
GL_TEXTURE_VIEW=_C('GL_TEXTURE_VIEW',0x82B5)
GL_TEXTURE_VIEW_MIN_LAYER=_C('GL_TEXTURE_VIEW_MIN_LAYER',0x82DD)
GL_TEXTURE_VIEW_MIN_LEVEL=_C('GL_TEXTURE_VIEW_MIN_LEVEL',0x82DB)
GL_TEXTURE_VIEW_NUM_LAYERS=_C('GL_TEXTURE_VIEW_NUM_LAYERS',0x82DE)
GL_TEXTURE_VIEW_NUM_LEVELS=_C('GL_TEXTURE_VIEW_NUM_LEVELS',0x82DC)
GL_TOP_LEVEL_ARRAY_SIZE=_C('GL_TOP_LEVEL_ARRAY_SIZE',0x930C)
GL_TOP_LEVEL_ARRAY_STRIDE=_C('GL_TOP_LEVEL_ARRAY_STRIDE',0x930D)
GL_TRANSFORM_FEEDBACK_VARYING=_C('GL_TRANSFORM_FEEDBACK_VARYING',0x92F4)
GL_TYPE=_C('GL_TYPE',0x92FA)
GL_UNIFORM=_C('GL_UNIFORM',0x92E1)
GL_UNIFORM_BLOCK=_C('GL_UNIFORM_BLOCK',0x92E2)
GL_UNIFORM_BLOCK_REFERENCED_BY_COMPUTE_SHADER=_C('GL_UNIFORM_BLOCK_REFERENCED_BY_COMPUTE_SHADER',0x90EC)
GL_VERTEX_ARRAY=_C('GL_VERTEX_ARRAY',0x8074)
GL_VERTEX_ATTRIB_ARRAY_LONG=_C('GL_VERTEX_ATTRIB_ARRAY_LONG',0x874E)
GL_VERTEX_ATTRIB_BINDING=_C('GL_VERTEX_ATTRIB_BINDING',0x82D4)
GL_VERTEX_ATTRIB_RELATIVE_OFFSET=_C('GL_VERTEX_ATTRIB_RELATIVE_OFFSET',0x82D5)
GL_VERTEX_BINDING_BUFFER=_C('GL_VERTEX_BINDING_BUFFER',0x8F4F)
GL_VERTEX_BINDING_DIVISOR=_C('GL_VERTEX_BINDING_DIVISOR',0x82D6)
GL_VERTEX_BINDING_OFFSET=_C('GL_VERTEX_BINDING_OFFSET',0x82D7)
GL_VERTEX_BINDING_STRIDE=_C('GL_VERTEX_BINDING_STRIDE',0x82D8)
GL_VERTEX_SUBROUTINE=_C('GL_VERTEX_SUBROUTINE',0x92E8)
GL_VERTEX_SUBROUTINE_UNIFORM=_C('GL_VERTEX_SUBROUTINE_UNIFORM',0x92EE)
GL_VERTEX_TEXTURE=_C('GL_VERTEX_TEXTURE',0x829B)
GL_VIEW_CLASS_128_BITS=_C('GL_VIEW_CLASS_128_BITS',0x82C4)
GL_VIEW_CLASS_16_BITS=_C('GL_VIEW_CLASS_16_BITS',0x82CA)
GL_VIEW_CLASS_24_BITS=_C('GL_VIEW_CLASS_24_BITS',0x82C9)
GL_VIEW_CLASS_32_BITS=_C('GL_VIEW_CLASS_32_BITS',0x82C8)
GL_VIEW_CLASS_48_BITS=_C('GL_VIEW_CLASS_48_BITS',0x82C7)
GL_VIEW_CLASS_64_BITS=_C('GL_VIEW_CLASS_64_BITS',0x82C6)
GL_VIEW_CLASS_8_BITS=_C('GL_VIEW_CLASS_8_BITS',0x82CB)
GL_VIEW_CLASS_96_BITS=_C('GL_VIEW_CLASS_96_BITS',0x82C5)
GL_VIEW_CLASS_BPTC_FLOAT=_C('GL_VIEW_CLASS_BPTC_FLOAT',0x82D3)
GL_VIEW_CLASS_BPTC_UNORM=_C('GL_VIEW_CLASS_BPTC_UNORM',0x82D2)
GL_VIEW_CLASS_RGTC1_RED=_C('GL_VIEW_CLASS_RGTC1_RED',0x82D0)
GL_VIEW_CLASS_RGTC2_RG=_C('GL_VIEW_CLASS_RGTC2_RG',0x82D1)
GL_VIEW_CLASS_S3TC_DXT1_RGB=_C('GL_VIEW_CLASS_S3TC_DXT1_RGB',0x82CC)
GL_VIEW_CLASS_S3TC_DXT1_RGBA=_C('GL_VIEW_CLASS_S3TC_DXT1_RGBA',0x82CD)
GL_VIEW_CLASS_S3TC_DXT3_RGBA=_C('GL_VIEW_CLASS_S3TC_DXT3_RGBA',0x82CE)
GL_VIEW_CLASS_S3TC_DXT5_RGBA=_C('GL_VIEW_CLASS_S3TC_DXT5_RGBA',0x82CF)
GL_VIEW_COMPATIBILITY_CLASS=_C('GL_VIEW_COMPATIBILITY_CLASS',0x82B6)
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLintptr,_cs.GLsizei)
def glBindVertexBuffer(bindingindex,buffer,offset,stride):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glClearBufferData(target,internalformat,format,type,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLintptr,_cs.GLsizeiptr,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glClearBufferSubData(target,internalformat,offset,size,format,type,data):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei)
def glCopyImageSubData(srcName,srcTarget,srcLevel,srcX,srcY,srcZ,dstName,dstTarget,dstLevel,dstX,dstY,dstZ,srcWidth,srcHeight,srcDepth):pass
@_f
@_p.types(None,_cs.GLDEBUGPROC,ctypes.c_void_p)
def glDebugMessageCallback(callback,userParam):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLsizei,arrays.GLuintArray,_cs.GLboolean)
def glDebugMessageControl(source,type,severity,count,ids,enabled):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLenum,_cs.GLsizei,arrays.GLcharArray)
def glDebugMessageInsert(source,type,id,severity,length,buf):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glDispatchCompute(num_groups_x,num_groups_y,num_groups_z):pass
@_f
@_p.types(None,_cs.GLintptr)
def glDispatchComputeIndirect(indirect):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint)
def glFramebufferParameteri(target,pname,param):pass
@_f
@_p.types(_cs.GLuint,_cs.GLuint,_cs.GLsizei,arrays.GLuintArray,arrays.GLuintArray,arrays.GLuintArray,arrays.GLuintArray,arrays.GLsizeiArray,arrays.GLcharArray)
def glGetDebugMessageLog(count,bufSize,sources,types,ids,severities,lengths,messageLog):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetFramebufferParameteriv(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLsizei,arrays.GLint64Array)
def glGetInternalformati64v(target,internalformat,pname,bufSize,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray)
def glGetObjectLabel(identifier,name,bufSize,length,label):pass
@_f
@_p.types(None,ctypes.c_void_p,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray)
def glGetObjectPtrLabel(ptr,bufSize,length,label):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLvoidpArray)
def glGetPointerv(pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetProgramInterfaceiv(program,programInterface,pname,params):pass
@_f
@_p.types(_cs.GLuint,_cs.GLuint,_cs.GLenum,arrays.GLcharArray)
def glGetProgramResourceIndex(program,programInterface,name):pass
@_f
@_p.types(_cs.GLint,_cs.GLuint,_cs.GLenum,arrays.GLcharArray)
def glGetProgramResourceLocation(program,programInterface,name):pass
@_f
@_p.types(_cs.GLint,_cs.GLuint,_cs.GLenum,arrays.GLcharArray)
def glGetProgramResourceLocationIndex(program,programInterface,name):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray)
def glGetProgramResourceName(program,programInterface,index,bufSize,length,name):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLuintArray,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLintArray)
def glGetProgramResourceiv(program,programInterface,index,propCount,props,bufSize,length,params):pass
@_f
@_p.types(None,_cs.GLuint)
def glInvalidateBufferData(buffer):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLintptr,_cs.GLsizeiptr)
def glInvalidateBufferSubData(buffer,offset,length):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,arrays.GLuintArray)
def glInvalidateFramebuffer(target,numAttachments,attachments):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,arrays.GLuintArray,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei)
def glInvalidateSubFramebuffer(target,numAttachments,attachments,x,y,width,height):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint)
def glInvalidateTexImage(texture,level):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei)
def glInvalidateTexSubImage(texture,level,xoffset,yoffset,zoffset,width,height,depth):pass
@_f
@_p.types(None,_cs.GLenum,ctypes.c_void_p,_cs.GLsizei,_cs.GLsizei)
def glMultiDrawArraysIndirect(mode,indirect,drawcount,stride):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,ctypes.c_void_p,_cs.GLsizei,_cs.GLsizei)
def glMultiDrawElementsIndirect(mode,type,indirect,drawcount,stride):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLcharArray)
def glObjectLabel(identifier,name,length,label):pass
@_f
@_p.types(None,ctypes.c_void_p,_cs.GLsizei,arrays.GLcharArray)
def glObjectPtrLabel(ptr,length,label):pass
@_f
@_p.types(None,)
def glPopDebugGroup():pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLcharArray)
def glPushDebugGroup(source,id,length,message):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glShaderStorageBlockBinding(program,storageBlockIndex,storageBlockBinding):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLintptr,_cs.GLsizeiptr)
def glTexBufferRange(target,internalformat,buffer,offset,size):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLboolean)
def glTexStorage2DMultisample(target,samples,internalformat,width,height,fixedsamplelocations):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLboolean)
def glTexStorage3DMultisample(target,samples,internalformat,width,height,depth,fixedsamplelocations):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glTextureView(texture,target,origtexture,internalformat,minlevel,numlevels,minlayer,numlayers):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint)
def glVertexAttribBinding(attribindex,bindingindex):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLboolean,_cs.GLuint)
def glVertexAttribFormat(attribindex,size,type,normalized,relativeoffset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLuint)
def glVertexAttribIFormat(attribindex,size,type,relativeoffset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLuint)
def glVertexAttribLFormat(attribindex,size,type,relativeoffset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint)
def glVertexBindingDivisor(bindingindex,divisor):pass



'''OpenGL extension OES.stencil8

This module customises the behaviour of the 
OpenGL.raw.GLES1.OES.stencil8 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension enables 8-bit stencil component as a valid
	render buffer storage format.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/stencil8.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.OES.stencil8 import *
from OpenGL.raw.GLES1.OES.stencil8 import _EXTENSION_NAME

def glInitStencil8OES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_framebuffer_multisample'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_framebuffer_multisample',error_checker=_errors._error_checker)
GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT=_C('GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE_EXT',0x8D56)
GL_MAX_SAMPLES_EXT=_C('GL_MAX_SAMPLES_EXT',0x8D57)
GL_RENDERBUFFER_SAMPLES_EXT=_C('GL_RENDERBUFFER_SAMPLES_EXT',0x8CAB)
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei)
def glRenderbufferStorageMultisampleEXT(target,samples,internalformat,width,height):pass



'''OpenGL extension OES.required_internalformat

This module customises the behaviour of the 
OpenGL.raw.GLES1.OES.required_internalformat to provide a more 
Python-friendly API

Overview (from the spec)
	
	The ES 1.1 API allows an implementation to store texture data internally
	with arbitrary precision, regardless of the format and type of the data
	supplied by the application.  Similarly, ES allows an implementation to
	choose an arbitrary precision for the internal storage of image data
	allocated by glRenderbufferStorageOES.
	
	While this allows flexibility for implementations, it does mean that an 
	application does not have a reliable means to request the implementation 
	maintain a specific precision or to find out what precision the 
	implementation will maintain for a given texture or renderbuffer image.
	
	For reference, "Desktop" OpenGL uses the <internalformat> argument to 
	glTexImage*, glCopyTexImage* and glRenderbufferStorageEXT as a hint, 
	defining the particular base format and precision that the application wants 
	the implementation to maintain when storing the image data.  Further, the 
	application can choose an <internalformat> with a different base internal 
	format than the source format specified by <format>.  The implementation is
	not required to exactly match the precision specified by <internalformat>
	when choosing an internal storage precision, but it is required to match the 
	base internal format of <internalformat>.
	
	In addition, ES 1.1 does not allow an implementation to fail a request to
	glTexImage2D for any of the legal <format> and <type> combinations listed in
	Table 3.4, even if the implementation does not natively support data stored
	in that external <format> and <type>.  However, there are no additional
	requirements placed on the implementation.  The ES implementation is free to
	store the texture data with lower precision than originally specified, for
	instance.  Further, since ES removes the ability to query the texture object
	to find out what internal format it chose, there is no way for the
	application to find out that this has happened.
	
	This extension addresses the situation in two ways:
	
	    1) This extension introduces the ability for an application to specify
	    the desired "sized" internal formats for texture image allocation.
	
	    2) This extension guarantees to maintain at least the specified
	    precision of all available sized internal formats.
	
	An implementation that exports this extension is committing to support all
	of the legal values for <internalformat> in Tables 3.4, 3.4.x, and 3.4.y,
	subject to the extension dependencies described herein.  That is to say, the 
	implementation is guaranteeing that choosing an <internalformat> argument
	with a value from these tables will not cause an image allocation request to
	fail.  Furthermore, it is guaranteeing that for any sized internal format,
	the renderbuffer or texture data will be stored with at least the precision
	prescribed by the sized internal format.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/required_internalformat.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.OES.required_internalformat import *
from OpenGL.raw.GLES1.OES.required_internalformat import _EXTENSION_NAME

def glInitRequiredInternalformatOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension SGIX.pixel_tiles

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.pixel_tiles to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension deals with the interaction of existing GL functions that
	read pixels from memory, applications that use grids of tiles of pixels, 
	and convolution.
	
	Applications that deal with large multi-dimensional images sometimes
	break the image into a grid of rectangular tiles of pixels. Such an
	approach can help control memory use and expedite roaming through an 
	image that is large with respect to the available memory. 
	
	GL functions that cause pixels to be read from memory (e.g., DrawPixels 
	and TexImage2D) assume the pixels are stored as a single series of rows
	of pixels. The grid of tiles is essentially a sequence of the structures
	that the pixel reading functions assume. When an application that uses
	tiling uses a GL function such as DrawPixels, it must iterate
	through the tiles, either coalescing the tiles into a single tile in
	preparation for a single GL call or calling the GL function for each tile.
	
	The convolution operation imposes strict ordering on the way pixels
	in a subimage that crosses tile boundaries must be transferred: the rows 
	of pixels transferred must span the entire subimage. Applications 
	that use tiles of pixels and convolution must copy the subimage to be 
	transferred from the grid of tiles to a contiguous region, then pass the
	now-contiguous rows of pixels to the convolution function. If the
	coalescing of tiles is not needed for some other reason or is not a 
	side effect of some necessary operation, it is just redundant movement 
	of the pixels.
	
	This extension seeks to eliminate the extra copy of data by extending the 
	existing GL functions to accept, as a source of pixels in memory, a 
	grid of tiles of pixels in addition to the current sequence of rows 
	of pixels.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/pixel_tiles.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.pixel_tiles import *
from OpenGL.raw.GL.SGIX.pixel_tiles import _EXTENSION_NAME

def glInitPixelTilesSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension INTEL.fragment_shader_ordering

This module customises the behaviour of the 
OpenGL.raw.GL.INTEL.fragment_shader_ordering to provide a more 
Python-friendly API

Overview (from the spec)
	
	Graphics devices may execute in parallel fragment shaders referring to the
	same window xy coordinates. Framebuffer writes are guaranteed to be
	processed in primitive rasterization order, but there is no order guarantee
	for other instructions and image or buffer object accesses in
	particular.
	
	The extension introduces a new GLSL built-in function, 
	beginFragmentShaderOrderingINTEL(), which blocks execution of a fragment 
	shader invocation until invocations from previous primitives that map to 
	the same xy window coordinates (and same sample when per-sample shading
	is active) complete their execution. All memory transactions from previous 
	fragment shader invocations are made visible to the fragment shader 
	invocation that called beginFragmentShaderOrderingINTEL() when the function 
	returns. 
	
	Including the following line in a shader can be used to control the
	language features described in this extension:
	
	  #extension GL_INTEL_fragment_shader_ordering : <behavior>
	
	where <behavior> is as specified in section 3.3.
	
	New preprocessor #defines are added to the OpenGL Shading Language:
	
	  #define GL_INTEL_fragment_shader_ordering 1

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/INTEL/fragment_shader_ordering.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.INTEL.fragment_shader_ordering import *
from OpenGL.raw.GL.INTEL.fragment_shader_ordering import _EXTENSION_NAME

def glInitFragmentShaderOrderingINTEL():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.command_list

This module customises the behaviour of the 
OpenGL.raw.GL.NV.command_list to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds a few new features designed to provide very low 
	overhead batching and replay of rendering commands and state changes:
	
	- A state object, which stores a pre-validated representation of the
	  the state of (almost) the entire pipeline.
	
	- A more flexible and extensible MultiDrawIndirect (MDI) type of mechanism, using
	  a token-based command stream, allowing to setup binding state and emit draw calls.
	
	- A set of functions to execute a list of the token-based command streams with state object
	  changes interleaved with the streams.
	
	- Command lists enabling compilation and reuse of sequences of command
	  streams and state object changes.
	
	Because state objects reflect the state of the entire pipeline, it is 
	expected that they can be pre-validated and executed efficiently. It is 
	also expected that when state objects are combined into a command list,
	the command list can diff consecutive state objects to produce a reduced/
	optimized set of state changes specific to that transition.
	
	The token-based command stream can also be stored in regular buffer objects
	and therefore be modified by the server itself. This allows more 
	complex work creation than the original MDI approach, which was limited
	to emitting draw calls only.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/command_list.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.command_list import *
from OpenGL.raw.GL.NV.command_list import _EXTENSION_NAME

def glInitCommandListNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glCreateStatesNV.states size not checked against n
glCreateStatesNV=wrapper.wrapper(glCreateStatesNV).setInputArraySize(
    'states', None
)
# INPUT glDeleteStatesNV.states size not checked against n
glDeleteStatesNV=wrapper.wrapper(glDeleteStatesNV).setInputArraySize(
    'states', None
)
# INPUT glCreateCommandListsNV.lists size not checked against n
glCreateCommandListsNV=wrapper.wrapper(glCreateCommandListsNV).setInputArraySize(
    'lists', None
)
# INPUT glDeleteCommandListsNV.lists size not checked against n
glDeleteCommandListsNV=wrapper.wrapper(glDeleteCommandListsNV).setInputArraySize(
    'lists', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.texture_storage

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.texture_storage to provide a more 
Python-friendly API

Overview (from the spec)
	
	The texture image specification commands in OpenGL allow each level
	to be separately specified with different sizes, formats, types and
	so on, and only imposes consistency checks at draw time. This adds
	overhead for implementations.
	
	This extension provides a mechanism for specifying the entire
	structure of a texture in a single call, allowing certain
	consistency checks and memory allocations to be done up front. Once
	specified, the format and dimensions of the image array become
	immutable, to simplify completeness checks in the implementation.
	
	When using this extension, it is no longer possible to supply texture
	data using TexImage*. Instead, data can be uploaded using TexSubImage*,
	or produced by other means (such as render-to-texture, mipmap generation,
	or rendering to a sibling EGLImage).
	
	This extension has complicated interactions with other extensions.
	The goal of most of these interactions is to ensure that a texture
	is always mipmap complete (and cube complete for cubemap textures).

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/texture_storage.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.texture_storage import *
from OpenGL.raw.GLES2.EXT.texture_storage import _EXTENSION_NAME

def glInitTextureStorageEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ANGLE.translated_shader_source

This module customises the behaviour of the 
OpenGL.raw.GLES2.ANGLE.translated_shader_source to provide a more 
Python-friendly API

Overview (from the spec)
	
	WebGL uses the GLSL ES 2.0 spec on all platforms, and translates these
	shaders to the host platform's native language (HLSL, GLSL, and even GLSL
	ES). For debugging purposes, it is useful to be able to examine the shader
	after translation.
	
	This extension addes a new function to query the translated shader source,
	and adds a new enum for GetShaderiv's <pname> parameter to query the
	translated shader source length. 

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ANGLE/translated_shader_source.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.ANGLE.translated_shader_source import *
from OpenGL.raw.GLES2.ANGLE.translated_shader_source import _EXTENSION_NAME

def glInitTranslatedShaderSourceANGLE():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glGetTranslatedShaderSourceANGLE=wrapper.wrapper(glGetTranslatedShaderSourceANGLE).setInputArraySize(
    'length', 1
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.shader_implicit_conversions

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.shader_implicit_conversions to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides support for implicitly converting signed integer
	types to unsigned types, as well as more general implicit conversion and
	function overloading infrastructure to support new data types introduced by
	other extensions.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/shader_implicit_conversions.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.shader_implicit_conversions import *
from OpenGL.raw.GLES2.EXT.shader_implicit_conversions import _EXTENSION_NAME

def glInitShaderImplicitConversionsEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.compute_shader_derivatives

This module customises the behaviour of the 
OpenGL.raw.GL.NV.compute_shader_derivatives to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds OpenGL and OpenGL ES API support for the OpenGL
	Shading Language (GLSL) extension "NV_compute_shader_derivatives".
	
	That extension, when enabled, allows applications to use derivatives in
	compute shaders.  It adds compute shader support for explicit derivative
	built-in functions like dFdx(), automatic derivative computation in
	texture lookup functions like texture(), use of the optional LOD bias
	parameter to adjust the computed level of detail values in texture lookup
	functions, and the texture level of detail query function
	textureQueryLod().

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/compute_shader_derivatives.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.compute_shader_derivatives import *
from OpenGL.raw.GL.NV.compute_shader_derivatives import _EXTENSION_NAME

def glInitComputeShaderDerivativesNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""EGL (cross-platform) platform library"""
import ctypes, ctypes.util
from OpenGL.platform import baseplatform, ctypesloader

class EGLPlatform( baseplatform.BasePlatform ):
    """EGL platform for opengl-es only platforms"""
    @baseplatform.lazy_property
    def GLES1(self):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'GLESv1_CM', # ick
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError:
            return None
    @baseplatform.lazy_property
    def GLES2(self):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'GLESv2', 
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError:
            return None
    @baseplatform.lazy_property
    def GLES3(self):
        # implementers guide says to use the same name for the DLL
        return self.GLES2
    @baseplatform.lazy_property
    def GL(self):
        try:
            for name in ('OpenGL','GL'):
                lib = ctypesloader.loadLibrary(
                    ctypes.cdll,
                    name, 
                    mode=ctypes.RTLD_GLOBAL 
                )
                if lib:
                    return lib 
            raise OSError("No GL/OpenGL library available")
        except OSError:
            return self.GLES2 or self.GLES1
    @baseplatform.lazy_property
    def GLU(self):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'GLU',
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError:
            return None
    @baseplatform.lazy_property
    def GLUT( self ):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'glut', 
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError:
            return None
    @baseplatform.lazy_property
    def OpenGL(self): return self.GL
    
    @baseplatform.lazy_property
    def EGL(self):
        # TODO: the raspberry pi crashes on trying to load EGL module 
        # because the EGL library requires a structure from GLES2 without 
        # linking to that library... Github issue is here:
        #   https://github.com/raspberrypi/firmware/issues/110
        import os
        if os.path.exists('/proc/cpuinfo'):
            info = open('/proc/cpuinfo').read()
            if 'BCM2708' in info or 'BCM2709' in info:
                assert self.GLES2
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'EGL', 
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError as err:
            raise ImportError("Unable to load EGL library", *err.args)
    @baseplatform.lazy_property
    def getExtensionProcedure( self ):
        eglGetProcAddress = self.EGL.eglGetProcAddress
        eglGetProcAddress.restype = ctypes.c_void_p
        return eglGetProcAddress
    @baseplatform.lazy_property
    def GLE( self ):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'gle', 
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError:
            return None

    DEFAULT_FUNCTION_TYPE = staticmethod( ctypes.CFUNCTYPE )
    @baseplatform.lazy_property
    def GetCurrentContext( self ):
        return self.EGL.eglGetCurrentContext



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C
# Spec mixes constants from 1.0 and 1.1
from OpenGL.raw.GL.VERSION.GL_1_0 import *
import ctypes
_EXTENSION_NAME = 'GL_VERSION_GL_1_1'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_VERSION_GL_1_1',error_checker=_errors._error_checker)
GL_ALPHA12=_C('GL_ALPHA12',0x803D)
GL_ALPHA16=_C('GL_ALPHA16',0x803E)
GL_ALPHA4=_C('GL_ALPHA4',0x803B)
GL_ALPHA8=_C('GL_ALPHA8',0x803C)
GL_C3F_V3F=_C('GL_C3F_V3F',0x2A24)
GL_C4F_N3F_V3F=_C('GL_C4F_N3F_V3F',0x2A26)
GL_C4UB_V2F=_C('GL_C4UB_V2F',0x2A22)
GL_C4UB_V3F=_C('GL_C4UB_V3F',0x2A23)
GL_CLIENT_ALL_ATTRIB_BITS=_C('GL_CLIENT_ALL_ATTRIB_BITS',0xFFFFFFFF)
GL_CLIENT_ATTRIB_STACK_DEPTH=_C('GL_CLIENT_ATTRIB_STACK_DEPTH',0x0BB1)
GL_CLIENT_PIXEL_STORE_BIT=_C('GL_CLIENT_PIXEL_STORE_BIT',0x00000001)
GL_CLIENT_VERTEX_ARRAY_BIT=_C('GL_CLIENT_VERTEX_ARRAY_BIT',0x00000002)
GL_COLOR_ARRAY=_C('GL_COLOR_ARRAY',0x8076)
GL_COLOR_ARRAY_POINTER=_C('GL_COLOR_ARRAY_POINTER',0x8090)
GL_COLOR_ARRAY_SIZE=_C('GL_COLOR_ARRAY_SIZE',0x8081)
GL_COLOR_ARRAY_STRIDE=_C('GL_COLOR_ARRAY_STRIDE',0x8083)
GL_COLOR_ARRAY_TYPE=_C('GL_COLOR_ARRAY_TYPE',0x8082)
GL_COLOR_LOGIC_OP=_C('GL_COLOR_LOGIC_OP',0x0BF2)
GL_DOUBLE=_C('GL_DOUBLE',0x140A)
GL_EDGE_FLAG_ARRAY=_C('GL_EDGE_FLAG_ARRAY',0x8079)
GL_EDGE_FLAG_ARRAY_POINTER=_C('GL_EDGE_FLAG_ARRAY_POINTER',0x8093)
GL_EDGE_FLAG_ARRAY_STRIDE=_C('GL_EDGE_FLAG_ARRAY_STRIDE',0x808C)
GL_FEEDBACK_BUFFER_POINTER=_C('GL_FEEDBACK_BUFFER_POINTER',0x0DF0)
GL_FEEDBACK_BUFFER_SIZE=_C('GL_FEEDBACK_BUFFER_SIZE',0x0DF1)
GL_FEEDBACK_BUFFER_TYPE=_C('GL_FEEDBACK_BUFFER_TYPE',0x0DF2)
GL_INDEX_ARRAY=_C('GL_INDEX_ARRAY',0x8077)
GL_INDEX_ARRAY_POINTER=_C('GL_INDEX_ARRAY_POINTER',0x8091)
GL_INDEX_ARRAY_STRIDE=_C('GL_INDEX_ARRAY_STRIDE',0x8086)
GL_INDEX_ARRAY_TYPE=_C('GL_INDEX_ARRAY_TYPE',0x8085)
GL_INDEX_LOGIC_OP=_C('GL_INDEX_LOGIC_OP',0x0BF1)
GL_INTENSITY=_C('GL_INTENSITY',0x8049)
GL_INTENSITY12=_C('GL_INTENSITY12',0x804C)
GL_INTENSITY16=_C('GL_INTENSITY16',0x804D)
GL_INTENSITY4=_C('GL_INTENSITY4',0x804A)
GL_INTENSITY8=_C('GL_INTENSITY8',0x804B)
GL_LUMINANCE12=_C('GL_LUMINANCE12',0x8041)
GL_LUMINANCE12_ALPHA12=_C('GL_LUMINANCE12_ALPHA12',0x8047)
GL_LUMINANCE12_ALPHA4=_C('GL_LUMINANCE12_ALPHA4',0x8046)
GL_LUMINANCE16=_C('GL_LUMINANCE16',0x8042)
GL_LUMINANCE16_ALPHA16=_C('GL_LUMINANCE16_ALPHA16',0x8048)
GL_LUMINANCE4=_C('GL_LUMINANCE4',0x803F)
GL_LUMINANCE4_ALPHA4=_C('GL_LUMINANCE4_ALPHA4',0x8043)
GL_LUMINANCE6_ALPHA2=_C('GL_LUMINANCE6_ALPHA2',0x8044)
GL_LUMINANCE8=_C('GL_LUMINANCE8',0x8040)
GL_LUMINANCE8_ALPHA8=_C('GL_LUMINANCE8_ALPHA8',0x8045)
GL_MAX_CLIENT_ATTRIB_STACK_DEPTH=_C('GL_MAX_CLIENT_ATTRIB_STACK_DEPTH',0x0D3B)
GL_N3F_V3F=_C('GL_N3F_V3F',0x2A25)
GL_NORMAL_ARRAY=_C('GL_NORMAL_ARRAY',0x8075)
GL_NORMAL_ARRAY_POINTER=_C('GL_NORMAL_ARRAY_POINTER',0x808F)
GL_NORMAL_ARRAY_STRIDE=_C('GL_NORMAL_ARRAY_STRIDE',0x807F)
GL_NORMAL_ARRAY_TYPE=_C('GL_NORMAL_ARRAY_TYPE',0x807E)
GL_POLYGON_OFFSET_FACTOR=_C('GL_POLYGON_OFFSET_FACTOR',0x8038)
GL_POLYGON_OFFSET_FILL=_C('GL_POLYGON_OFFSET_FILL',0x8037)
GL_POLYGON_OFFSET_LINE=_C('GL_POLYGON_OFFSET_LINE',0x2A02)
GL_POLYGON_OFFSET_POINT=_C('GL_POLYGON_OFFSET_POINT',0x2A01)
GL_POLYGON_OFFSET_UNITS=_C('GL_POLYGON_OFFSET_UNITS',0x2A00)
GL_PROXY_TEXTURE_1D=_C('GL_PROXY_TEXTURE_1D',0x8063)
GL_PROXY_TEXTURE_2D=_C('GL_PROXY_TEXTURE_2D',0x8064)
GL_R3_G3_B2=_C('GL_R3_G3_B2',0x2A10)
GL_RGB10=_C('GL_RGB10',0x8052)
GL_RGB10_A2=_C('GL_RGB10_A2',0x8059)
GL_RGB12=_C('GL_RGB12',0x8053)
GL_RGB16=_C('GL_RGB16',0x8054)
GL_RGB4=_C('GL_RGB4',0x804F)
GL_RGB5=_C('GL_RGB5',0x8050)
GL_RGB5_A1=_C('GL_RGB5_A1',0x8057)
GL_RGB8=_C('GL_RGB8',0x8051)
GL_RGBA12=_C('GL_RGBA12',0x805A)
GL_RGBA16=_C('GL_RGBA16',0x805B)
GL_RGBA2=_C('GL_RGBA2',0x8055)
GL_RGBA4=_C('GL_RGBA4',0x8056)
GL_RGBA8=_C('GL_RGBA8',0x8058)
GL_SELECTION_BUFFER_POINTER=_C('GL_SELECTION_BUFFER_POINTER',0x0DF3)
GL_SELECTION_BUFFER_SIZE=_C('GL_SELECTION_BUFFER_SIZE',0x0DF4)
GL_T2F_C3F_V3F=_C('GL_T2F_C3F_V3F',0x2A2A)
GL_T2F_C4F_N3F_V3F=_C('GL_T2F_C4F_N3F_V3F',0x2A2C)
GL_T2F_C4UB_V3F=_C('GL_T2F_C4UB_V3F',0x2A29)
GL_T2F_N3F_V3F=_C('GL_T2F_N3F_V3F',0x2A2B)
GL_T2F_V3F=_C('GL_T2F_V3F',0x2A27)
GL_T4F_C4F_N3F_V4F=_C('GL_T4F_C4F_N3F_V4F',0x2A2D)
GL_T4F_V4F=_C('GL_T4F_V4F',0x2A28)
GL_TEXTURE_ALPHA_SIZE=_C('GL_TEXTURE_ALPHA_SIZE',0x805F)
GL_TEXTURE_BINDING_1D=_C('GL_TEXTURE_BINDING_1D',0x8068)
GL_TEXTURE_BINDING_2D=_C('GL_TEXTURE_BINDING_2D',0x8069)
GL_TEXTURE_BLUE_SIZE=_C('GL_TEXTURE_BLUE_SIZE',0x805E)
GL_TEXTURE_COORD_ARRAY=_C('GL_TEXTURE_COORD_ARRAY',0x8078)
GL_TEXTURE_COORD_ARRAY_POINTER=_C('GL_TEXTURE_COORD_ARRAY_POINTER',0x8092)
GL_TEXTURE_COORD_ARRAY_SIZE=_C('GL_TEXTURE_COORD_ARRAY_SIZE',0x8088)
GL_TEXTURE_COORD_ARRAY_STRIDE=_C('GL_TEXTURE_COORD_ARRAY_STRIDE',0x808A)
GL_TEXTURE_COORD_ARRAY_TYPE=_C('GL_TEXTURE_COORD_ARRAY_TYPE',0x8089)
GL_TEXTURE_GREEN_SIZE=_C('GL_TEXTURE_GREEN_SIZE',0x805D)
GL_TEXTURE_INTENSITY_SIZE=_C('GL_TEXTURE_INTENSITY_SIZE',0x8061)
GL_TEXTURE_INTERNAL_FORMAT=_C('GL_TEXTURE_INTERNAL_FORMAT',0x1003)
GL_TEXTURE_LUMINANCE_SIZE=_C('GL_TEXTURE_LUMINANCE_SIZE',0x8060)
GL_TEXTURE_PRIORITY=_C('GL_TEXTURE_PRIORITY',0x8066)
GL_TEXTURE_RED_SIZE=_C('GL_TEXTURE_RED_SIZE',0x805C)
GL_TEXTURE_RESIDENT=_C('GL_TEXTURE_RESIDENT',0x8067)
GL_V2F=_C('GL_V2F',0x2A20)
GL_V3F=_C('GL_V3F',0x2A21)
GL_VERTEX_ARRAY=_C('GL_VERTEX_ARRAY',0x8074)
GL_VERTEX_ARRAY_POINTER=_C('GL_VERTEX_ARRAY_POINTER',0x808E)
GL_VERTEX_ARRAY_SIZE=_C('GL_VERTEX_ARRAY_SIZE',0x807A)
GL_VERTEX_ARRAY_STRIDE=_C('GL_VERTEX_ARRAY_STRIDE',0x807C)
GL_VERTEX_ARRAY_TYPE=_C('GL_VERTEX_ARRAY_TYPE',0x807B)
@_f
@_p.types(_cs.GLboolean,_cs.GLsizei,arrays.GLuintArray,arrays.GLbooleanArray)
def glAreTexturesResident(n,textures,residences):pass
@_f
@_p.types(None,_cs.GLint)
def glArrayElement(i):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glBindTexture(target,texture):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glColorPointer(size,type,stride,pointer):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLint)
def glCopyTexImage1D(target,level,internalformat,x,y,width,border):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLint)
def glCopyTexImage2D(target,level,internalformat,x,y,width,height,border):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei)
def glCopyTexSubImage1D(target,level,xoffset,x,y,width):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei)
def glCopyTexSubImage2D(target,level,xoffset,yoffset,x,y,width,height):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glDeleteTextures(n,textures):pass
@_f
@_p.types(None,_cs.GLenum)
def glDisableClientState(array):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLsizei)
def glDrawArrays(mode,first,count):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,ctypes.c_void_p)
def glDrawElements(mode,count,type,indices):pass
@_f
@_p.types(None,_cs.GLsizei,ctypes.c_void_p)
def glEdgeFlagPointer(stride,pointer):pass
@_f
@_p.types(None,_cs.GLenum)
def glEnableClientState(array):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glGenTextures(n,textures):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLvoidpArray)
def glGetPointerv(pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glIndexPointer(type,stride,pointer):pass
@_f
@_p.types(None,_cs.GLubyte)
def glIndexub(c):pass
@_f
@_p.types(None,arrays.GLubyteArray)
def glIndexubv(c):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glInterleavedArrays(format,stride,pointer):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint)
def glIsTexture(texture):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glNormalPointer(type,stride,pointer):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat)
def glPolygonOffset(factor,units):pass
@_f
@_p.types(None,)
def glPopClientAttrib():pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray,arrays.GLfloatArray)
def glPrioritizeTextures(n,textures,priorities):pass
@_f
@_p.types(None,_cs.GLbitfield)
def glPushClientAttrib(mask):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glTexCoordPointer(size,type,stride,pointer):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glTexSubImage1D(target,level,xoffset,width,format,type,pixels):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLenum,_cs.GLenum,ctypes.c_void_p)
def glTexSubImage2D(target,level,xoffset,yoffset,width,height,format,type,pixels):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glVertexPointer(size,type,stride,pointer):pass



'''OpenGL extension EXT.shader_texture_lod

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.shader_texture_lod to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds additional texture functions to the
	OpenGL ES Shading Language which provide the shader writer
	with explicit control of LOD.
	
	Mipmap texture fetches and anisotropic texture fetches
	require implicit derivatives to calculate rho, lambda
	and/or the line of anisotropy.  These implicit derivatives
	will be undefined for texture fetches occurring inside
	non-uniform control flow or for vertex shader texture
	fetches, resulting in undefined texels.
	
	The additional texture functions introduced with
	this extension provide explicit control of LOD
	(isotropic texture functions) or provide explicit
	derivatives (anisotropic texture functions).
	
	Anisotropic texture functions return defined texels
	for mipmap texture fetches or anisotropic texture fetches,
	even inside non-uniform control flow.  Isotropic texture
	functions return defined texels for mipmap texture fetches,
	even inside non-uniform control flow.  However, isotropic
	texture functions return undefined texels for anisotropic
	texture fetches.
	
	The existing isotropic vertex texture functions:
	
	    vec4 texture2DLodEXT(sampler2D sampler,
	                         vec2 coord, 
	                         float lod);
	    vec4 texture2DProjLodEXT(sampler2D sampler,
	                             vec3 coord, 
	                             float lod);
	    vec4 texture2DProjLodEXT(sampler2D sampler,
	                             vec4 coord, 
	                             float lod);
	
	    vec4 textureCubeLodEXT(samplerCube sampler,
	                           vec3 coord,
	                           float lod);
	
	are added to the built-in functions for fragment shaders
	with "EXT" suffix appended.
	
	New anisotropic texture functions, providing explicit
	derivatives:
	
	    vec4 texture2DGradEXT(sampler2D sampler,
	                          vec2 P, 
	                          vec2 dPdx, 
	                          vec2  dPdy);
	    vec4 texture2DProjGradEXT(sampler2D sampler,
	                              vec3 P, 
	                              vec2 dPdx, 
	                              vec2 dPdy);
	    vec4 texture2DProjGradEXT(sampler2D sampler,
	                              vec4 P,
	                              vec2 dPdx, 
	                              vec2 dPdy);
	
	    vec4 textureCubeGradEXT(samplerCube sampler,
	                            vec3 P,
	                            vec3 dPdx, 
	                            vec3 dPdy);
	
	 are added to the built-in functions for vertex shaders
	 and fragment shaders.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/shader_texture_lod.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.shader_texture_lod import *
from OpenGL.raw.GLES2.EXT.shader_texture_lod import _EXTENSION_NAME

def glInitShaderTextureLodEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_EXT_memory_object'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_EXT_memory_object',error_checker=_errors._error_checker)
GL_DEDICATED_MEMORY_OBJECT_EXT=_C('GL_DEDICATED_MEMORY_OBJECT_EXT',0x9581)
GL_DEVICE_UUID_EXT=_C('GL_DEVICE_UUID_EXT',0x9597)
GL_DRIVER_UUID_EXT=_C('GL_DRIVER_UUID_EXT',0x9598)
GL_LINEAR_TILING_EXT=_C('GL_LINEAR_TILING_EXT',0x9585)
GL_NUM_DEVICE_UUIDS_EXT=_C('GL_NUM_DEVICE_UUIDS_EXT',0x9596)
GL_NUM_TILING_TYPES_EXT=_C('GL_NUM_TILING_TYPES_EXT',0x9582)
GL_OPTIMAL_TILING_EXT=_C('GL_OPTIMAL_TILING_EXT',0x9584)
GL_PROTECTED_MEMORY_OBJECT_EXT=_C('GL_PROTECTED_MEMORY_OBJECT_EXT',0x959B)
GL_TEXTURE_TILING_EXT=_C('GL_TEXTURE_TILING_EXT',0x9580)
GL_TILING_TYPES_EXT=_C('GL_TILING_TYPES_EXT',0x9583)
GL_UUID_SIZE_EXT=_C('GL_UUID_SIZE_EXT',16)
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizeiptr,_cs.GLuint,_cs.GLuint64)
def glBufferStorageMemEXT(target,size,memory,offset):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glCreateMemoryObjectsEXT(n,memoryObjects):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glDeleteMemoryObjectsEXT(n,memoryObjects):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetMemoryObjectParameterivEXT(memoryObject,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLubyteArray)
def glGetUnsignedBytei_vEXT(target,index,data):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLubyteArray)
def glGetUnsignedBytevEXT(pname,data):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint)
def glIsMemoryObjectEXT(memoryObject):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glMemoryObjectParameterivEXT(memoryObject,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizeiptr,_cs.GLuint,_cs.GLuint64)
def glNamedBufferStorageMemEXT(buffer,size,memory,offset):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLuint,_cs.GLuint64)
def glTexStorageMem1DEXT(target,levels,internalFormat,width,memory,offset):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLuint,_cs.GLuint64)
def glTexStorageMem2DEXT(target,levels,internalFormat,width,height,memory,offset):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLboolean,_cs.GLuint,_cs.GLuint64)
def glTexStorageMem2DMultisampleEXT(target,samples,internalFormat,width,height,fixedSampleLocations,memory,offset):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLuint,_cs.GLuint64)
def glTexStorageMem3DEXT(target,levels,internalFormat,width,height,depth,memory,offset):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLboolean,_cs.GLuint,_cs.GLuint64)
def glTexStorageMem3DMultisampleEXT(target,samples,internalFormat,width,height,depth,fixedSampleLocations,memory,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLuint,_cs.GLuint64)
def glTextureStorageMem1DEXT(texture,levels,internalFormat,width,memory,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLuint,_cs.GLuint64)
def glTextureStorageMem2DEXT(texture,levels,internalFormat,width,height,memory,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLboolean,_cs.GLuint,_cs.GLuint64)
def glTextureStorageMem2DMultisampleEXT(texture,samples,internalFormat,width,height,fixedSampleLocations,memory,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLuint,_cs.GLuint64)
def glTextureStorageMem3DEXT(texture,levels,internalFormat,width,height,depth,memory,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLboolean,_cs.GLuint,_cs.GLuint64)
def glTextureStorageMem3DMultisampleEXT(texture,samples,internalFormat,width,height,depth,fixedSampleLocations,memory,offset):pass



"""Debug utilities for EGL operations"""
from OpenGL.EGL import *
import itertools


def eglErrorName(value):
    """Returns error constant if known, otherwise returns value"""
    return KNOWN_ERRORS.get(value, value)


KNOWN_ERRORS = {
    EGL_SUCCESS: EGL_SUCCESS,
    EGL_NOT_INITIALIZED: EGL_NOT_INITIALIZED,
    EGL_BAD_ACCESS: EGL_BAD_ACCESS,
    EGL_BAD_ALLOC: EGL_BAD_ALLOC,
    EGL_BAD_ATTRIBUTE: EGL_BAD_ATTRIBUTE,
    EGL_BAD_CONTEXT: EGL_BAD_CONTEXT,
    EGL_BAD_CONFIG: EGL_BAD_CONFIG,
    EGL_BAD_CURRENT_SURFACE: EGL_BAD_CURRENT_SURFACE,
    EGL_BAD_DISPLAY: EGL_BAD_DISPLAY,
    EGL_BAD_SURFACE: EGL_BAD_SURFACE,
    EGL_BAD_MATCH: EGL_BAD_MATCH,
    EGL_BAD_PARAMETER: EGL_BAD_PARAMETER,
    EGL_BAD_NATIVE_PIXMAP: EGL_BAD_NATIVE_PIXMAP,
    EGL_BAD_NATIVE_WINDOW: EGL_BAD_NATIVE_WINDOW,
    EGL_CONTEXT_LOST: EGL_CONTEXT_LOST,
}


def write_ppm(buf, filename):
    """Write height * width * 3-component buffer as ppm to filename
    
    This lets us write a simple image format without
    using any libraries that can be viewed on most
    linux workstations.
    """
    with open(filename, "w") as f:
        (h, w, c) = buf.shape
        f.write("P3\n")
        f.write("# ascii ppm file created by pyopengl\n")
        f.write("%i %i\n" % (w, h))
        f.write("255\n")
        for y in range(h - 1, -1, -1):
            for x in range(w):
                pixel = buf[y, x]
                l = " %3d %3d %3d" % (pixel[0], pixel[1], pixel[2])
                f.write(l)
            f.write("\n")


def debug_config(display, config):
    """Get debug display for the given configuration"""
    result = {}
    value = EGLint()
    for attr in CONFIG_ATTRS:
        if not eglGetConfigAttrib(display, config, attr, value):
            log.warning("Failed to get attribute %s from config", attr)
            continue
        if attr in BITMASK_FIELDS:
            attr_value = {}
            for subattr in BITMASK_FIELDS[attr]:
                if value.value & subattr:
                    attr_value[subattr.name] = True
        else:
            attr_value = value.value
        result[attr.name] = attr_value
    return result


def debug_configs(display, configs=None, max_count=256):
    """Present a formatted list of configs for the display"""
    if configs is None:
        configs = (EGLConfig * max_count)()
        num_configs = EGLint()
        eglGetConfigs(display, configs, max_count, num_configs)
        if not num_configs.value:
            return []
        configs = configs[: num_configs.value]
    debug_configs = [debug_config(display, cfg) for cfg in configs]
    return debug_configs


SURFACE_TYPE_BITS = [
    EGL_MULTISAMPLE_RESOLVE_BOX_BIT,
    EGL_PBUFFER_BIT,
    EGL_PIXMAP_BIT,
    EGL_SWAP_BEHAVIOR_PRESERVED_BIT,
    EGL_VG_ALPHA_FORMAT_PRE_BIT,
    EGL_VG_COLORSPACE_LINEAR_BIT,
    EGL_WINDOW_BIT,
]
RENDERABLE_TYPE_BITS = [
    EGL_OPENGL_BIT,
    EGL_OPENGL_ES_BIT,
    EGL_OPENGL_ES2_BIT,
    EGL_OPENGL_ES3_BIT,
    EGL_OPENVG_BIT,
]
CAVEAT_BITS = [
    EGL_NONE,
    EGL_SLOW_CONFIG,
    EGL_NON_CONFORMANT_CONFIG,
]
TRANSPARENT_BITS = [
    EGL_NONE,
    EGL_TRANSPARENT_RGB,
]

CONFIG_ATTRS = [
    EGL_CONFIG_ID,
    EGL_RED_SIZE,
    EGL_GREEN_SIZE,
    EGL_BLUE_SIZE,
    EGL_DEPTH_SIZE,
    EGL_ALPHA_SIZE,
    EGL_ALPHA_MASK_SIZE,
    EGL_BUFFER_SIZE,
    EGL_STENCIL_SIZE,
    EGL_BIND_TO_TEXTURE_RGB,
    EGL_BIND_TO_TEXTURE_RGBA,
    EGL_COLOR_BUFFER_TYPE,
    EGL_CONFIG_CAVEAT,
    EGL_CONFORMANT,
    EGL_LEVEL,
    EGL_LUMINANCE_SIZE,
    EGL_MAX_PBUFFER_WIDTH,
    EGL_MAX_PBUFFER_HEIGHT,
    EGL_MAX_PBUFFER_PIXELS,
    EGL_MIN_SWAP_INTERVAL,
    EGL_MAX_SWAP_INTERVAL,
    EGL_NATIVE_RENDERABLE,
    EGL_NATIVE_VISUAL_ID,
    EGL_NATIVE_VISUAL_TYPE,
    EGL_RENDERABLE_TYPE,
    EGL_SAMPLE_BUFFERS,
    EGL_SAMPLES,
    EGL_SURFACE_TYPE,
    EGL_TRANSPARENT_TYPE,
    EGL_TRANSPARENT_RED_VALUE,
    EGL_TRANSPARENT_GREEN_VALUE,
    EGL_TRANSPARENT_BLUE_VALUE,
]

BITMASK_FIELDS = dict(
    [
        (EGL_SURFACE_TYPE, SURFACE_TYPE_BITS),
        (EGL_RENDERABLE_TYPE, RENDERABLE_TYPE_BITS),
        (EGL_CONFORMANT, RENDERABLE_TYPE_BITS),
        (EGL_CONFIG_CAVEAT, CAVEAT_BITS),
        (EGL_TRANSPARENT_TYPE, TRANSPARENT_BITS),
    ]
)


def bit_renderer(bit):
    def render(value):
        if bit.name in value:
            return " Y"
        else:
            return " ."

    return render


CONFIG_FORMAT = [
    (EGL_CONFIG_ID, "0x%x", "id", "cfg"),
    (EGL_BUFFER_SIZE, "%i", "sz", "bf"),
    (EGL_LEVEL, "%i", "l", "lv"),
    (EGL_RED_SIZE, "%i", "r", "cbuf"),
    (EGL_GREEN_SIZE, "%i", "g", "cbuf"),
    (EGL_BLUE_SIZE, "%i", "b", "cbuf"),
    (EGL_ALPHA_SIZE, "%i", "a", "cbuf"),
    (EGL_DEPTH_SIZE, "%i", "th", "dp"),
    (EGL_STENCIL_SIZE, "%i", "t", "s"),
    (EGL_SAMPLES, "%i", "ns", "mult"),
    (EGL_SAMPLE_BUFFERS, "%i", "bu", "mult"),
    (EGL_NATIVE_VISUAL_ID, "0x%x", "id", "visual"),
    (EGL_RENDERABLE_TYPE, bit_renderer(EGL_OPENGL_BIT), "gl", "render"),
    (EGL_RENDERABLE_TYPE, bit_renderer(EGL_OPENGL_ES_BIT), "es", "render"),
    (EGL_RENDERABLE_TYPE, bit_renderer(EGL_OPENGL_ES2_BIT), "e2", "render"),
    (EGL_RENDERABLE_TYPE, bit_renderer(EGL_OPENGL_ES3_BIT), "e3", "render"),
    (EGL_RENDERABLE_TYPE, bit_renderer(EGL_OPENVG_BIT), "vg", "render"),
    (EGL_SURFACE_TYPE, bit_renderer(EGL_WINDOW_BIT), "wn", "surface"),
    (EGL_SURFACE_TYPE, bit_renderer(EGL_PBUFFER_BIT), "pb", "surface"),
    (EGL_SURFACE_TYPE, bit_renderer(EGL_PIXMAP_BIT), "px", "surface"),
]


def format_debug_configs(debug_configs, formats=CONFIG_FORMAT):
    """Format config for compact debugging display
    
    Produces a config summary display for a set of 
    debug_configs as a text-mode table.

    Uses `formats` (default `CONFIG_FORMAT`) to determine 
    which fields are extracted and how they are formatted
    along with the column/subcolum set to be rendered in
    the overall header.

    returns formatted ASCII table for display in debug
    logs or utilities
    """
    columns = []
    for (key, format, subcol, col) in formats:
        column = []
        max_width = 0
        for row in debug_configs:
            if isinstance(row, EGLConfig):
                raise TypeError(row, "Call debug_config(display,config)")
            try:
                value = row[key.name]
            except KeyError:
                formatted = "_"
            else:
                if isinstance(format, str):
                    formatted = format % (value)
                else:
                    formatted = format(value)
            max_width = max((len(formatted), max_width))
            column.append(formatted)
        columns.append(
            {
                "rows": column,
                "key": key,
                "format": format,
                "subcol": subcol,
                "col": col,
                "width": max_width,
            }
        )
    headers = []
    subheaders = []
    rows = [headers, subheaders]
    last_column = None
    last_column_width = 0
    for header, subcols in itertools.groupby(columns, lambda x: x["col"]):
        subcols = list(subcols)
        width = sum([col["width"] for col in subcols]) + (len(subcols) - 1)
        headers.append(header.center(width, ".")[:width])
    for column in columns:
        subheaders.append(column["subcol"].rjust(column["width"])[: column["width"]])
    rows.extend(
        zip(*[[v.rjust(col["width"], " ") for v in col["rows"]] for col in columns])
    )
    return "\n".join([" ".join(row) for row in rows])



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES1 import _types as _cs
# End users want this...
from OpenGL.raw.GLES1._types import *
from OpenGL.raw.GLES1 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES1_IMG_texture_env_enhanced_fixed_function'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES1,'GLES1_IMG_texture_env_enhanced_fixed_function',error_checker=_errors._error_checker)
GL_ADD_BLEND_IMG=_C('GL_ADD_BLEND_IMG',0x8C09)
GL_DOT3_RGBA_IMG=_C('GL_DOT3_RGBA_IMG',0x86AF)
GL_FACTOR_ALPHA_MODULATE_IMG=_C('GL_FACTOR_ALPHA_MODULATE_IMG',0x8C07)
GL_FRAGMENT_ALPHA_MODULATE_IMG=_C('GL_FRAGMENT_ALPHA_MODULATE_IMG',0x8C08)
GL_MODULATE_COLOR_IMG=_C('GL_MODULATE_COLOR_IMG',0x8C04)
GL_RECIP_ADD_SIGNED_ALPHA_IMG=_C('GL_RECIP_ADD_SIGNED_ALPHA_IMG',0x8C05)
GL_TEXTURE_ALPHA_MODULATE_IMG=_C('GL_TEXTURE_ALPHA_MODULATE_IMG',0x8C06)




'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_NV_memory_attachment'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_NV_memory_attachment',error_checker=_errors._error_checker)
GL_ATTACHED_MEMORY_OBJECT_NV=_C('GL_ATTACHED_MEMORY_OBJECT_NV',0x95A4)
GL_ATTACHED_MEMORY_OFFSET_NV=_C('GL_ATTACHED_MEMORY_OFFSET_NV',0x95A5)
GL_DETACHED_BUFFERS_NV=_C('GL_DETACHED_BUFFERS_NV',0x95AB)
GL_DETACHED_MEMORY_INCARNATION_NV=_C('GL_DETACHED_MEMORY_INCARNATION_NV',0x95A9)
GL_DETACHED_TEXTURES_NV=_C('GL_DETACHED_TEXTURES_NV',0x95AA)
GL_MAX_DETACHED_BUFFERS_NV=_C('GL_MAX_DETACHED_BUFFERS_NV',0x95AD)
GL_MAX_DETACHED_TEXTURES_NV=_C('GL_MAX_DETACHED_TEXTURES_NV',0x95AC)
GL_MEMORY_ATTACHABLE_ALIGNMENT_NV=_C('GL_MEMORY_ATTACHABLE_ALIGNMENT_NV',0x95A6)
GL_MEMORY_ATTACHABLE_NV=_C('GL_MEMORY_ATTACHABLE_NV',0x95A8)
GL_MEMORY_ATTACHABLE_SIZE_NV=_C('GL_MEMORY_ATTACHABLE_SIZE_NV',0x95A7)
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint64)
def glBufferAttachMemoryNV(target,memory,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLint,_cs.GLsizei,arrays.GLuintArray)
def glGetMemoryObjectDetachedResourcesuivNV(memory,pname,first,count,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint64)
def glNamedBufferAttachMemoryNV(buffer,memory,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum)
def glResetMemoryObjectParameterNV(memory,pname):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint64)
def glTexAttachMemoryNV(target,memory,offset):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint64)
def glTextureAttachMemoryNV(texture,memory,offset):pass



'''OpenGL extension OES.depth24

This module customises the behaviour of the 
OpenGL.raw.GLSC2.OES.depth24 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension enables 24-bit depth components as a valid
	render buffer storage format.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/depth24.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLSC2 import _types, _glgets
from OpenGL.raw.GLSC2.OES.depth24 import *
from OpenGL.raw.GLSC2.OES.depth24 import _EXTENSION_NAME

def glInitDepth24OES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension OES.texture_float

This module customises the behaviour of the 
OpenGL.raw.GLES2.OES.texture_float to provide a more 
Python-friendly API

Overview (from the spec)
	
	These extensions add texture formats with 16- (aka half float) and 32-bit
	floating-point components.  The 32-bit floating-point components
	are in the standard IEEE float format.  The 16-bit floating-point
	components have 1 sign bit, 5 exponent bits, and 10 mantissa bits.
	Floating-point components are clamped to the limits of the range
	representable by their format.
	
	The OES_texture_half_float extension string indicates that the
	implementation supports 16-bit floating pt texture formats.
	
	The OES_texture_float extension string indicates that the 
	implementation supports 32-bit floating pt texture formats.
	
	Both these extensions only require NEAREST magnification filter and 
	NEAREST, and NEAREST_MIPMAP_NEAREST minification filters to be supported.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/texture_float.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.OES.texture_float import *
from OpenGL.raw.GLES2.OES.texture_float import _EXTENSION_NAME

def glInitTextureFloatOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ANGLE.framebuffer_blit

This module customises the behaviour of the 
OpenGL.raw.GLES2.ANGLE.framebuffer_blit to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension modifies framebuffer objects by splitting the
	framebuffer object binding point into separate DRAW and READ
	bindings.  This allows copying directly from one framebuffer to
	another.  In addition, a new high performance blit function is
	added to facilitate these blits and perform some data conversion
	where allowed.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ANGLE/framebuffer_blit.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.ANGLE.framebuffer_blit import *
from OpenGL.raw.GLES2.ANGLE.framebuffer_blit import _EXTENSION_NAME

def glInitFramebufferBlitANGLE():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension APPLE.clip_distance

This module customises the behaviour of the 
OpenGL.raw.GLES2.APPLE.clip_distance to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds support for hardware clip planes to OpenGL ES 2.0
	and 3.0.  These were present in OpenGL ES 1.1, but were removed to
	better match certain hardware.  Since they're useful for certain
	applications, notable CAD, we return them here.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/APPLE/clip_distance.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.APPLE.clip_distance import *
from OpenGL.raw.GLES2.APPLE.clip_distance import _EXTENSION_NAME

def glInitClipDistanceAPPLE():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension SGIX.calligraphic_fragment

This module customises the behaviour of the 
OpenGL.raw.GL.SGIX.calligraphic_fragment to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a minimal mechanism to control the copying of
	fragment information to external hardware such as a calligraphic
	display interface.  The initial implementation is intended to support
	the calligraphic interface on InfiniteReality.  On InfiniteReality
	when the interface is enabled, fragment information consisting of
	some color and coverage data is sent to the interface.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/calligraphic_fragment.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIX.calligraphic_fragment import *
from OpenGL.raw.GL.SGIX.calligraphic_fragment import _EXTENSION_NAME

def glInitCalligraphicFragmentSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.imaging

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.imaging to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/imaging.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.imaging import *
from OpenGL.raw.GL.ARB.imaging import _EXTENSION_NAME

def glInitImagingARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glColorTable.table size not checked against 'format,type,width'
glColorTable=wrapper.wrapper(glColorTable).setInputArraySize(
    'table', None
)
# INPUT glColorTableParameterfv.params size not checked against 'pname'
glColorTableParameterfv=wrapper.wrapper(glColorTableParameterfv).setInputArraySize(
    'params', None
)
# INPUT glColorTableParameteriv.params size not checked against 'pname'
glColorTableParameteriv=wrapper.wrapper(glColorTableParameteriv).setInputArraySize(
    'params', None
)
# OUTPUT glGetColorTable.table COMPSIZE(target, format, type) 
glGetColorTableParameterfv=wrapper.wrapper(glGetColorTableParameterfv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetColorTableParameteriv=wrapper.wrapper(glGetColorTableParameteriv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# INPUT glColorSubTable.data size not checked against 'format,type,count'
glColorSubTable=wrapper.wrapper(glColorSubTable).setInputArraySize(
    'data', None
)
# INPUT glConvolutionFilter1D.image size not checked against 'format,type,width'
glConvolutionFilter1D=wrapper.wrapper(glConvolutionFilter1D).setInputArraySize(
    'image', None
)
# INPUT glConvolutionFilter2D.image size not checked against 'format,type,width,height'
glConvolutionFilter2D=wrapper.wrapper(glConvolutionFilter2D).setInputArraySize(
    'image', None
)
# INPUT glConvolutionParameterfv.params size not checked against 'pname'
glConvolutionParameterfv=wrapper.wrapper(glConvolutionParameterfv).setInputArraySize(
    'params', None
)
# INPUT glConvolutionParameteriv.params size not checked against 'pname'
glConvolutionParameteriv=wrapper.wrapper(glConvolutionParameteriv).setInputArraySize(
    'params', None
)
# OUTPUT glGetConvolutionFilter.image COMPSIZE(target, format, type) 
glGetConvolutionParameterfv=wrapper.wrapper(glGetConvolutionParameterfv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetConvolutionParameteriv=wrapper.wrapper(glGetConvolutionParameteriv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# OUTPUT glGetSeparableFilter.column COMPSIZE(target, format, type) 
# OUTPUT glGetSeparableFilter.row COMPSIZE(target, format, type) 
# OUTPUT glGetSeparableFilter.span COMPSIZE(target, format, type) 
# INPUT glSeparableFilter2D.column size not checked against 'target,format,type,height'
# INPUT glSeparableFilter2D.row size not checked against 'target,format,type,width'
glSeparableFilter2D=wrapper.wrapper(glSeparableFilter2D).setInputArraySize(
    'column', None
).setInputArraySize(
    'row', None
)
# OUTPUT glGetHistogram.values COMPSIZE(target, format, type) 
glGetHistogramParameterfv=wrapper.wrapper(glGetHistogramParameterfv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetHistogramParameteriv=wrapper.wrapper(glGetHistogramParameteriv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# OUTPUT glGetMinmax.values COMPSIZE(target, format, type) 
glGetMinmaxParameterfv=wrapper.wrapper(glGetMinmaxParameterfv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetMinmaxParameteriv=wrapper.wrapper(glGetMinmaxParameteriv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
### END AUTOGENERATED SECTION
from OpenGL.GL import images
from OpenGL.lazywrapper import lazy as _lazy
glColorTable = images.setDimensionsAsInts(
    images.setImageInput(
        glColorTable,
        pixelName = 'table',
        typeName = 'type',
    )
)
glColorSubTable = images.setDimensionsAsInts(
    images.setImageInput(
        glColorSubTable,
        pixelName = 'data',
    )
)
glSeparableFilter2D = images.setDimensionsAsInts(
    images.setImageInput(
        images.setImageInput(
            glSeparableFilter2D,
            pixelName = 'row',
            typeName = 'type',
        ),
        pixelName = 'column',
        typeName = 'type',
    )
)
glConvolutionFilter1D = images.setDimensionsAsInts(
    images.setImageInput(
        glConvolutionFilter1D,
        pixelName = 'image',
        typeName = 'type',
    )
)
glConvolutionFilter2D = images.setDimensionsAsInts(
    images.setImageInput(
        glConvolutionFilter2D,
        pixelName = 'image',
        typeName = 'type',
    )
)

@_lazy( glGetConvolutionFilter )
def glGetConvolutionFilter( baseFunction, target, format, type ):
    """Retrieve 1 or 2D convolution parameter "kernels" as pixel data"""
    dims = (
        glGetConvolutionParameteriv( target, GL_CONVOLUTION_WIDTH )[0],
    )
    if target != GL_CONVOLUTION_1D:
        dims += (
            glGetConvolutionParameteriv( target, GL_CONVOLUTION_HEIGHT )[0],
        )
    # is it always 4?  Seems to be, but the spec/man-page isn't really clear about it...
    dims += (4,)
    array = images.images.SetupPixelRead( format, dims, type )
    arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[
        images.images.TYPE_TO_ARRAYTYPE.get(type,type)
    ]
    baseFunction(
        target, format, type,
        ctypes.c_void_p( arrayType.dataPointer(array))
    )
    return array
@_lazy( glGetSeparableFilter )
def glGetSeparableFilter( baseFunction, target, format, type ):
    """Retrieve 2 1D convolution parameter "kernels" as pixel data"""
    rowDims = (
        glGetConvolutionParameteriv( GL_CONVOLUTION_WIDTH )[0],
        4,
    )
    columnDims = (
        glGetConvolutionParameteriv( GL_CONVOLUTION_HEIGHT )[0],
        4,
    )
    arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[
        images.images.TYPE_TO_ARRAYTYPE.get(type,type)
    ]
    row = images.images.SetupPixelRead( format, rowDims, type )
    column = images.images.SetupPixelRead( format, columnDims, type )
    baseFunction(
        target, format, type,
        ctypes.c_void_p( arrayType.dataPointer(row)),
        ctypes.c_void_p( arrayType.dataPointer(column)),
        None # span
    )
    return row, column
@_lazy( glGetColorTable )
def glGetColorTable( baseFunction, target, format, type ):
    """Retrieve the current 1D color table as a bitmap"""
    dims = (
        glGetColorTableParameteriv(target, GL_COLOR_TABLE_WIDTH),
        4, # Grr, spec *seems* to say that it's different sizes, but it doesn't really say...
    )
    array = images.images.SetupPixelRead( format, dims, type )
    arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[
        images.images.TYPE_TO_ARRAYTYPE.get(type,type)
    ]
    baseFunction(
        target, format, type,
        ctypes.c_void_p( arrayType.dataPointer(array))
    )
    return array
@_lazy( glGetHistogram )
def glGetHistogram( baseFunction, target, reset, format, type, values=None):
    """Retrieve current 1D histogram as a 1D bitmap"""
    if values is None:
        width = glGetHistogramParameteriv(
            target,
            GL_HISTOGRAM_WIDTH,
        )
        values = images.images.SetupPixelRead( format, (width,4), type )
    arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[
        images.images.TYPE_TO_ARRAYTYPE.get(type,type)
    ]
    baseFunction(
        target, reset, format, type,
        ctypes.c_void_p( arrayType.dataPointer(values))
    )
    return values

@_lazy( glGetMinmax )
def glGetMinmax( baseFunction, target, reset, format, type, values=None):
    """Retrieve minimum and maximum values as a 2-element image"""
    if values is None:
        width = 2
        values = images.images.SetupPixelRead( format, (width,4), type )
    arrayType = arrays.GL_CONSTANT_TO_ARRAY_TYPE[
        images.images.TYPE_TO_ARRAYTYPE.get(type,type)
    ]
    baseFunction(
        target, reset, format, type,
        ctypes.c_void_p( arrayType.dataPointer(values))
    )
    return values



'''OpenGL extension IMG.multisampled_render_to_texture

This module customises the behaviour of the 
OpenGL.raw.GLES1.IMG.multisampled_render_to_texture to provide a more 
Python-friendly API

Overview (from the spec)
	
	    This extension introduces functionality to perform multisampled 
		rendering to a color renderable texture, without requiring an 
		explicit resolve of multisample data. 
	
		Some GPU architectures - such as tile-based renderers - are
		capable of performing multisampled rendering by storing 
		multisample data in internal high-speed memory and downsampling the
		data when writing out to external memory after rendering has 
		finished. Since per-sample data is never written out to external 
		memory, this approach saves bandwidth and storage space. In this 
		case multisample data gets discarded, however this is acceptable
		in most cases.
	
		The extension provides a new command, FramebufferTexture2DMultisampleIMG, 
		which attaches a texture level to a framebuffer and enables 
		multisampled rendering to that texture level. 
	
		When the texture level is used as a source or destination for any 
		operation other than drawing to it, an implicit resolve of 
		multisampled color data is performed. After such a resolve, the 
		multisampled color data is discarded.
	
		In order to allow the use of multisampled depth and stencil buffers 
		when performing	multisampled rendering to a texture, the extension 
		also adds the command RenderbufferStorageMultisampleIMG.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/IMG/multisampled_render_to_texture.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.IMG.multisampled_render_to_texture import *
from OpenGL.raw.GLES1.IMG.multisampled_render_to_texture import _EXTENSION_NAME

def glInitMultisampledRenderToTextureIMG():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension AMD.compressed_ATC_texture

This module customises the behaviour of the 
OpenGL.raw.GLES1.AMD.compressed_ATC_texture to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension enables support for ATC compressed texture formats.  ATC is 
	AMD's proprietary compression algorithm for compressing textures for 
	handheld devices to save on power consumption, memory footprint and 
	bandwidth.
	
	Three compression formats are introduced:
	
	- A compression format for RGB textures.
	- A compression format for RGBA textures using explicit alpha encoding.
	- A compression format for RGBA textures using interpolated alpha encoding.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/compressed_ATC_texture.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.AMD.compressed_ATC_texture import *
from OpenGL.raw.GLES1.AMD.compressed_ATC_texture import _EXTENSION_NAME

def glInitCompressedAtcTextureAMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION